#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import standard library
import abc

# Import modules
import pytest


class ABCTestTopology(abc.ABC):
    """Abstract class that defines various tests for topologies

    Whenever a topology inherits from ABCTestTopology,
    you don't need to write down all tests anymore. Instead, you can just
    specify all required fixtures in the test suite.
    """

    @pytest.fixture
    def topology(self):
        """Return an instance of the topology"""
        raise NotImplementedError("NotImplementedError::topology")

    @pytest.fixture
    def options(self):
        """Return a dictionary of options"""
        raise NotImplementedError("NotImplementedError::options")

    @pytest.mark.parametrize("static", [True, False])
    @pytest.mark.parametrize("clamp", [None, (0, 1), (-1, 1)])
    def test_compute_velocity_return_values(
        self, topology, swarm, clamp, static
    ):
        """Test if compute_velocity() gives the expected shape and range"""
        topo = topology(static=static)
        v = topo.compute_velocity(swarm, clamp)
        assert v.shape == swarm.position.shape
        if clamp is not None:
            assert (clamp[0] <= v).all() and (clamp[1] >= v).all()

    @pytest.mark.parametrize("static", [True, False])
    @pytest.mark.parametrize(
        "bounds",
        [None, ([-5, -5, -5], [5, 5, 5]), ([-10, -10, -10], [10, 10, 10])],
    )
    def test_compute_position_return_values(
        self, topology, swarm, bounds, static
    ):
        """Test if compute_position() gives the expected shape and range"""
        topo = topology(static=static)
        p = topo.compute_position(swarm, bounds)
        assert p.shape == swarm.velocity.shape
        if bounds is not None:
            assert (bounds[0] <= p).all() and (bounds[1] >= p).all()

    @pytest.mark.parametrize("static", [True, False])
    def test_neighbor_idx(self, topology, options, swarm, static):
        """Test if the neighbor_idx attribute is assigned"""
        topo = topology(static=static)
        topo.compute_gbest(swarm, **options)
        assert topo.neighbor_idx is not None

    @pytest.mark.parametrize("static", [True, False])
    @pytest.mark.parametrize("swarm", [0, (1, 2, 3)])
    def test_input_swarm(self, topology, static, swarm, options):
        """Test if AttributeError is raised when passed with a non-Swarm instance"""
        with pytest.raises(AttributeError):
            topo = topology(static=static)
            topo.compute_gbest(swarm, **options)



# -*- coding: utf-8 -*-
"""
Hyperparameter random search.

Compares the relative performance of combinations of randomly generated
hyperparameter values in optimizing a specified objective function.

User provides lists of bounds for the uniform random value generation of
'c1', 'c2', and 'w', and the random integer value generation of 'k'.
Combinations of values are generated for the number of iterations specified,
and the generated grid of combinations is used in the search method to find
the optimal parameters for the objective function. The search method default
returns the minimum objective function score and hyperparameters that yield
the minimum score, yet maximum score can also be evaluated.


>>> options = {'c1': [1, 5],
               'c2': [6, 10],
               'w' : [2, 5],
               'k' : [11, 15],
               'p' : 1}
>>> g = RandomSearch(LocalBestPSO, n_particles=40, dimensions=20,
                   options=options, objective_func=sphere, iters=10)
>>> best_score, best_options = g.search()
>>> best_score
1.41978545901
>>> best_options['c1']
1.543556887693
>>> best_options['c2']
9.504769054771
"""

# Import from __future__
from __future__ import absolute_import, print_function, with_statement

# Import modules
import numpy as np
from past.builtins import xrange

# Import from pyswarms
# Import from package
from pyswarms.utils.search.base_search import SearchBase


class RandomSearch(SearchBase):
    """Search of optimal performance on selected objective function
    over combinations of randomly selected hyperparameter values
    within specified bounds for specified number of selection iterations."""

    def assertions(self):
        """Assertion method to check :code:`n_selection_iters` input

        Raises
        ------
        TypeError
            When :code:`n_selection_iters` is not of type int
        """
        super(RandomSearch, self).assertions()

        # Check type of n_selection_iters parameter
        if not isinstance(self.n_selection_iters, int):
            raise TypeError(
                "Parameter `n_selection_iters` must be of " "type int"
            )

    def __init__(
        self,
        optimizer,
        n_particles,
        dimensions,
        options,
        objective_func,
        iters,
        n_selection_iters,
        bounds=None,
        velocity_clamp=(0, 1),
    ):
        """Initialize the Search

        Attributes
        ----------
        n_selection_iters: int
            number of iterations of random parameter selection
        """

        # Assign n_selection_iters as attribute
        self.n_selection_iters = n_selection_iters
        # Assign attributes
        super(RandomSearch, self).__init__(
            optimizer,
            n_particles,
            dimensions,
            options,
            objective_func,
            iters,
            bounds=bounds,
            velocity_clamp=velocity_clamp,
        )
        # Invoke assertions
        self.assertions()

    def generate_grid(self):
        """Generate the grid of hyperparameter value combinations"""

        options = dict(self.options)
        params = {}

        # Remove 'p' to hold as a constant in the paramater combinations
        p = options.pop("p")
        params["p"] = [p for _ in xrange(self.n_selection_iters)]

        # Assign generators based on parameter type
        param_generators = {
            "c1": np.random.uniform,
            "c2": np.random.uniform,
            "w": np.random.uniform,
            "k": np.random.randint,
        }

        # Generate random values for hyperparameters 'c1', 'c2', 'w', and 'k'
        for idx, bounds in options.items():
            params[idx] = param_generators[idx](
                *bounds, size=self.n_selection_iters
            )

        # Return list of dicts of hyperparameter combinations
        return [
            {
                "c1": params["c1"][i],
                "c2": params["c2"][i],
                "w": params["w"][i],
                "k": params["k"][i],
                "p": params["p"][i],
            }
            for i in xrange(self.n_selection_iters)
        ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import standard library
import abc

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.utils.functions.single_obj import rosenbrock, sphere


class ABCTestOptimizer(abc.ABC):
    """Abstract class that defines various tests for high-level optimizers

    Whenever an optimizer implementation inherits from ABCTestOptimizer,
    you don't need to write down all tests anymore. Instead, you can just
    specify all required fixtures in the test suite.
    """

    @pytest.fixture
    def optimizer(self):
        """Return an instance of the optimizer"""
        raise NotImplementedError("NotImplementedError::optimizer")

    @pytest.fixture
    def optimizer_history(self):
        """Run the optimizer for 1000 iterations and return its instance"""
        raise NotImplementedError("NotImplementedError::optimizer_history")

    @pytest.fixture
    def optimizer_reset(self):
        """Reset the optimizer and return its instance"""
        raise NotImplementedError("NotImplementedError::optimizer_reset")

    @pytest.fixture
    def options(self):
        """Default options dictionary for most PSO use-cases"""
        return {"c1": 0.3, "c2": 0.7, "w": 0.9, "k": 2, "p": 2, "r": 1}

    @pytest.fixture
    def obj_with_args(self):
        """Objective function with arguments"""

        def obj_with_args_(x, a, b):
            f = (a - x[:, 0]) ** 2 + b * (x[:, 1] - x[:, 0] ** 2) ** 2
            return f

        return obj_with_args_

    @pytest.fixture
    def obj_without_args(self):
        """Objective function without arguments"""
        return rosenbrock

    @pytest.mark.parametrize(
        "history, expected_shape",
        [
            ("cost_history", (1000,)),
            ("mean_pbest_history", (1000,)),
            ("mean_neighbor_history", (1000,)),
            ("pos_history", (1000, 10, 2)),
            ("velocity_history", (1000, 10, 2)),
        ],
    )
    def test_train_history(self, optimizer_history, history, expected_shape):
        """Test if training histories are of expected shape"""
        opt = vars(optimizer_history)
        assert np.array(opt[history]).shape == expected_shape

    def test_reset_default_values(self, optimizer_reset):
        """Test if best cost and best pos are set properly when the reset()
        method is called"""
        assert optimizer_reset.swarm.best_cost == np.inf
        assert set(optimizer_reset.swarm.best_pos) == set(np.array([]))

    @pytest.mark.skip(reason="The Ring topology converges too slowly")
    def test_ftol_effect(self, options, optimizer):
        """Test if setting the ftol breaks the optimization process"""
        opt = optimizer(10, 2, options=options, ftol=1e-1)
        opt.optimize(sphere, 2000)
        assert np.array(opt.cost_history).shape != (2000,)

    def test_parallel_evaluation(self, obj_without_args, optimizer, options):
        """Test if parallelization breaks the optimization process"""
        import multiprocessing
        opt = optimizer(100, 2, options=options)
        opt.optimize(obj_without_args, 2000, n_processes=multiprocessing.cpu_count())
        assert np.array(opt.cost_history).shape == (2000,)

    def test_obj_with_kwargs(self, obj_with_args, optimizer, options):
        """Test if kwargs are passed properly in objfunc"""
        x_max = 10 * np.ones(2)
        x_min = -1 * x_max
        bounds = (x_min, x_max)
        opt = optimizer(100, 2, options=options, bounds=bounds)
        cost, pos = opt.optimize(obj_with_args, 1000, a=1, b=100)
        assert np.isclose(cost, 0, rtol=1e-03)
        assert np.isclose(pos[0], 1.0, rtol=1e-03)
        assert np.isclose(pos[1], 1.0, rtol=1e-03)

    def test_obj_unnecessary_kwargs(
        self, obj_without_args, optimizer, options
    ):
        """Test if error is raised given unnecessary kwargs"""
        x_max = 10 * np.ones(2)
        x_min = -1 * x_max
        bounds = (x_min, x_max)
        opt = optimizer(100, 2, options=options, bounds=bounds)
        with pytest.raises(TypeError):
            # kwargs `a` should not be supplied
            cost, pos = opt.optimize(obj_without_args, 1000, a=1)

    def test_obj_missing_kwargs(self, obj_with_args, optimizer, options):
        """Test if error is raised with incomplete kwargs"""
        x_max = 10 * np.ones(2)
        x_min = -1 * x_max
        bounds = (x_min, x_max)
        opt = optimizer(100, 2, options=options, bounds=bounds)
        with pytest.raises(TypeError):
            # kwargs `b` is missing here
            cost, pos = opt.optimize(obj_with_args, 1000, a=1)

    def test_obj_incorrect_kwargs(self, obj_with_args, optimizer, options):
        """Test if error is raised with wrong kwargs"""
        x_max = 10 * np.ones(2)
        x_min = -1 * x_max
        bounds = (x_min, x_max)
        opt = optimizer(100, 2, options=options, bounds=bounds)
        with pytest.raises(TypeError):
            # Wrong kwargs
            cost, pos = opt.optimize(obj_with_args, 1000, c=1, d=100)



"""
The :mod:`pyswarms.base` module implements base
swarm classes to implement variants of particle swarm optimization.
"""

from .base_single import SwarmOptimizer
from .base_discrete import DiscreteSwarmOptimizer

__all__ = ["SwarmOptimizer" "DiscreteSwarmOptimizer"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import from standard library
import os

# Import modules
import pytest
import matplotlib as mpl

# Set $DISPLAY environmental variable
if os.environ.get("DISPLAY", "") == "":
    print("No display found. Using non-interactive Agg backend.")
    mpl.use("Agg")

from matplotlib.animation import FuncAnimation
from matplotlib.axes._subplots import SubplotBase

# Import from pyswarms
from pyswarms.utils.plotters import (
    plot_contour,
    plot_cost_history,
    plot_surface,
)
from pyswarms.utils.plotters.plotters import _animate, _mesh


@pytest.mark.parametrize(
    "history", ["cost_history", "mean_neighbor_history", "mean_pbest_history"]
)
def test_plot_cost_history_return_type(trained_optimizer, history):
    """Tests if plot_cost_history() returns a SubplotBase instance"""
    opt_params = vars(trained_optimizer)
    plot = plot_cost_history(opt_params[history])
    assert isinstance(plot, SubplotBase)


@pytest.mark.parametrize("bad_values", [2, 43.14])
def test_plot_cost_history_error(bad_values):
    """Tests if plot_cost_history() raises an error given bad values"""
    with pytest.raises(TypeError):
        plot_cost_history(bad_values)


def test_plot_contour_return_type(pos_history):
    """Tests if the animation function returns the expected type"""
    assert isinstance(plot_contour(pos_history), FuncAnimation)


def test_plot_surface_return_type(pos_history):
    """Tests if the animation function returns the expected type"""
    assert isinstance(plot_surface(pos_history), FuncAnimation)


def test_mesh_hidden_function_shape(mesher):
    """Tests if the hidden _mesh() function returns the expected shape"""
    xx, yy, zz = _mesh(mesher)
    assert xx.shape == yy.shape == zz.shape == (20, 20)


def test_animate_hidden_function_type(pos_history):
    """Tests if the hidden _animate() function returns the expected type"""
    fig, ax = mpl.pyplot.subplots(1, 1)
    ax = mpl.pyplot.scatter(x=[], y=[])
    return_plot = _animate(i=1, data=pos_history, plot=ax)
    assert isinstance(return_plot, tuple)



# -*- coding: utf-8 -*-

r"""
A Local-best Particle Swarm Optimization (lbest PSO) algorithm.

Similar to global-best PSO, it takes a set of candidate solutions,
and finds the best solution using a position-velocity update method.
However, it uses a ring topology, thus making the particles
attracted to its corresponding neighborhood.

The position update can be defined as:

.. math::

   x_{i}(t+1) = x_{i}(t) + v_{i}(t+1)

Where the position at the current timestep :math:`t` is updated using
the computed velocity at :math:`t+1`. Furthermore, the velocity update
is defined as:

.. math::

   v_{ij}(t + 1) = m * v_{ij}(t) + c_{1}r_{1j}(t)[y_{ij}(t) − x_{ij}(t)] + c_{2}r_{2j}(t)[\hat{y}_{j}(t) − x_{ij}(t)]

However, in local-best PSO, a particle doesn't compare itself to the
overall performance of the swarm. Instead, it looks at the performance
of its nearest-neighbours, and compares itself with them. In general,
this kind of topology takes much more time to converge, but has a more
powerful explorative feature.

In this implementation, a neighbor is selected via a k-D tree
imported from :code:`scipy`. Distance are computed with either
the L1 or L2 distance. The nearest-neighbours are then queried from
this k-D tree. They are computed for every iteration.

An example usage is as follows:

.. code-block:: python

    import pyswarms as ps
    from pyswarms.utils.functions import single_obj as fx

    # Set-up hyperparameters
    options = {'c1': 0.5, 'c2': 0.3, 'w': 0.9, 'k': 3, 'p': 2}

    # Call instance of LBestPSO with a neighbour-size of 3 determined by
    # the L2 (p=2) distance.
    optimizer = ps.single.LocalBestPSO(n_particles=10, dimensions=2,
                                       options=options)

    # Perform optimization
    stats = optimizer.optimize(fx.sphere, iters=100)

This algorithm was adapted from one of the earlier works of
J. Kennedy and R.C. Eberhart in Particle Swarm Optimization
[IJCNN1995]_ [MHS1995]_

.. [IJCNN1995] J. Kennedy and R.C. Eberhart, "Particle Swarm Optimization,"
    Proceedings of the IEEE International Joint Conference on Neural
    Networks, 1995, pp. 1942-1948.

.. [MHS1995] J. Kennedy and R.C. Eberhart, "A New Optimizer using Particle
    Swarm Theory,"  in Proceedings of the Sixth International
    Symposium on Micromachine and Human Science, 1995, pp. 39–43.
"""

# Import standard library
import logging

# Import modules
import numpy as np
import multiprocessing as mp

from ..backend.operators import compute_pbest, compute_objective_function
from ..backend.topology import Ring
from ..backend.handlers import BoundaryHandler, VelocityHandler
from ..base import SwarmOptimizer
from ..utils.reporter import Reporter


class LocalBestPSO(SwarmOptimizer):
    def __init__(
        self,
        n_particles,
        dimensions,
        options,
        bounds=None,
        bh_strategy="periodic",
        velocity_clamp=None,
        vh_strategy="unmodified",
        center=1.00,
        ftol=-np.inf,
        init_pos=None,
        static=False,
    ):
        """Initialize the swarm

        Attributes
        ----------
        n_particles : int
            number of particles in the swarm.
        dimensions : int
            number of dimensions in the space.
        bounds : tuple of numpy.ndarray
            a tuple of size 2 where the first entry is the minimum bound
            while the second entry is the maximum bound. Each array must
            be of shape :code:`(dimensions,)`.
        bh_strategy : str
            a strategy for the handling of out-of-bounds particles.
        velocity_clamp : tuple (default is :code:`(0,1)`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        vh_strategy : str
            a strategy for the handling of the velocity of out-of-bounds particles.
        center : list, optional
            an array of size :code:`dimensions`
        ftol : float
            relative error in objective_func(best_pos) acceptable for
            convergence. Default is :code:`-np.inf`
        options : dict with keys :code:`{'c1', 'c2', 'w', 'k', 'p'}`
            a dictionary containing the parameters for the specific
            optimization technique
                * c1 : float
                    cognitive parameter
                * c2 : float
                    social parameter
                * w : float
                    inertia parameter
                * k : int
                    number of neighbors to be considered. Must be a
                    positive integer less than :code:`n_particles`
                * p: int {1,2}
                    the Minkowski p-norm to use. 1 is the
                    sum-of-absolute values (or L1 distance) while 2 is
                    the Euclidean (or L2) distance.
        init_pos : numpy.ndarray, optional
            option to explicitly set the particles' initial positions. Set to
            :code:`None` if you wish to generate the particles randomly.
        static: bool
            a boolean that decides whether the Ring topology
            used is static or dynamic. Default is `False`
        """
        # Initialize logger
        self.logger = logging.getLogger(__name__)
        # Assign k-neighbors and p-value as attributes
        self.k, self.p = options["k"], options["p"]
        # Initialize parent class
        super(LocalBestPSO, self).__init__(
            n_particles=n_particles,
            dimensions=dimensions,
            options=options,
            bounds=bounds,
            velocity_clamp=velocity_clamp,
            center=center,
            ftol=ftol,
            init_pos=init_pos,
        )
        # Initialize logger
        self.rep = Reporter(logger=logging.getLogger(__name__))
        # Initialize the resettable attributes
        self.reset()
        # Initialize the topology
        self.top = Ring(static=static)
        self.bh = BoundaryHandler(strategy=bh_strategy)
        self.vh = VelocityHandler(strategy=vh_strategy)
        self.name = __name__

    def optimize(self, objective_func, iters, n_processes=None, **kwargs):
        """Optimize the swarm for a number of iterations

        Performs the optimization to evaluate the objective
        function :code:`f` for a number of iterations :code:`iter.`

        Parameters
        ----------
        objective_func : callable
            objective function to be evaluated
        iters : int
            number of iterations
        n_processes : int
            number of processes to use for parallel particle evaluation (default: None = no parallelization)
        kwargs : dict
            arguments for the objective function

        Returns
        -------
        tuple
            the local best cost and the local best position among the
            swarm.
        """
        self.rep.log("Obj. func. args: {}".format(kwargs), lvl=logging.DEBUG)
        self.rep.log(
            "Optimize for {} iters with {}".format(iters, self.options),
            lvl=logging.INFO,
        )
        # Populate memory of the handlers
        self.bh.memory = self.swarm.position
        self.vh.memory = self.swarm.position

        # Setup Pool of processes for parallel evaluation
        pool = None if n_processes is None else mp.Pool(n_processes)

        self.swarm.pbest_cost = np.full(self.swarm_size[0], np.inf)
        for i in self.rep.pbar(iters, self.name):
            # Compute cost for current position and personal best
            self.swarm.current_cost = compute_objective_function(
                self.swarm, objective_func, pool=pool, **kwargs
            )
            self.swarm.pbest_pos, self.swarm.pbest_cost = compute_pbest(
                self.swarm
            )
            best_cost_yet_found = np.min(self.swarm.best_cost)
            # Update gbest from neighborhood
            self.swarm.best_pos, self.swarm.best_cost = self.top.compute_gbest(
                self.swarm, p=self.p, k=self.k
            )
            self.rep.hook(best_cost=np.min(self.swarm.best_cost))
            # Save to history
            hist = self.ToHistory(
                best_cost=self.swarm.best_cost,
                mean_pbest_cost=np.mean(self.swarm.pbest_cost),
                mean_neighbor_cost=np.mean(self.swarm.best_cost),
                position=self.swarm.position,
                velocity=self.swarm.velocity,
            )
            self._populate_history(hist)
            # Verify stop criteria based on the relative acceptable cost ftol
            relative_measure = self.ftol * (1 + np.abs(best_cost_yet_found))
            if (
                np.abs(self.swarm.best_cost - best_cost_yet_found)
                < relative_measure
            ):
                break
            # Perform position velocity update
            self.swarm.velocity = self.top.compute_velocity(
                self.swarm, self.velocity_clamp, self.vh, self.bounds
            )
            self.swarm.position = self.top.compute_position(
                self.swarm, self.bounds, self.bh
            )
        # Obtain the final best_cost and the final best_position
        final_best_cost = self.swarm.best_cost.copy()
        final_best_pos = self.swarm.pbest_pos[self.swarm.pbest_cost.argmin()].copy()
        # Write report in log and return final cost and position
        self.rep.log(
            "Optimization finished | best cost: {}, best pos: {}".format(
                final_best_cost, final_best_pos
            ),
            lvl=logging.INFO,
        )
        return (final_best_cost, final_best_pos)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.backend.topology import Ring
from .abc_test_topology import ABCTestTopology

np.random.seed(4135157)


class TestRingTopology(ABCTestTopology):
    @pytest.fixture
    def topology(self):
        return Ring

    @pytest.fixture(params=[(1, 2), (2, 3)])
    def options(self, request):
        p, k = request.param
        return {"p": p, "k": k}

    @pytest.mark.parametrize("static", [True, False])
    @pytest.mark.parametrize("k", [i for i in range(1, 10)])
    @pytest.mark.parametrize("p", [1, 2])
    def test_compute_gbest_return_values(self, swarm, topology, p, k, static):
        """Test if update_gbest_neighborhood gives the expected return values"""
        topo = topology(static=static)
        pos, cost = topo.compute_gbest(swarm, p=p, k=k)
        expected_cost = 1.0002528364353296
        expected_pos = np.array([9.90438476e-01, 2.50379538e-03, 1.87405987e-05])
        expected_pos_2 = np.array([9.98033031e-01, 4.97392619e-03, 3.07726256e-03])
        assert cost == pytest.approx(expected_cost)
        assert ((pos[np.argmin(cost)] == pytest.approx(expected_pos)) or
                (pos[np.argmin(cost)] == pytest.approx(expected_pos_2)))



# -*- coding: utf-8 -*-

"""
A Pyramid Network Topology

This class implements a pyramid topology. In this topology, the particles are connected by N-dimensional simplices.
"""

# Import standard library
import logging

# Import modules
import numpy as np
from scipy.spatial import Delaunay

from .. import operators as ops
from ..handlers import BoundaryHandler, VelocityHandler
from ...utils.reporter import Reporter
from .base import Topology


class Pyramid(Topology):
    def __init__(self, static=False):
        """Initialize the class

        Parameters
        ----------
        static : bool (Default is :code:`False`)
            a boolean that decides whether the topology
            is static or dynamic
        """
        super(Pyramid, self).__init__(static)
        self.rep = Reporter(logger=logging.getLogger(__name__))

    def compute_gbest(self, swarm, **kwargs):
        """Update the global best using a pyramid neighborhood approach

        This topology uses the :code:`Delaunay` class from :code:`scipy`. To
        prevent precision errors in the Delaunay class, custom
        :code:`qhull_options` were added. Namely, :code:`QJ0.001 Qbb Qc Qx`.
        The meaning of those options is explained in [qhull]. This method is
        used to triangulate N-dimensional space into simplices. The vertices of
        the simplicies consist of swarm particles. This method is adapted from
        the work of Lane et al.[SIS2008]

        [SIS2008] J. Lane, A. Engelbrecht and J. Gain, "Particle swarm optimization with spatially
        meaningful neighbours," 2008 IEEE Swarm Intelligence Symposium, St. Louis, MO, 2008,
        pp. 1-8. doi: 10.1109/SIS.2008.4668281
        [qhull] http://www.qhull.org/html/qh-optq.htm

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance

        Returns
        -------
        numpy.ndarray
            Best position of shape :code:`(n_dimensions, )`
        float
            Best cost
        """
        try:
            # If there are less than (swarm.dimensions + 1) particles they are all connected
            if swarm.n_particles < swarm.dimensions + 1:
                self.neighbor_idx = np.tile(
                    np.arange(swarm.n_particles), (swarm.n_particles, 1)
                )
                best_pos = swarm.pbest_pos[np.argmin(swarm.pbest_cost)]
                best_cost = np.min(swarm.pbest_cost)
            else:
                # Check if the topology is static or dynamic and assign neighbors
                if (
                    self.static and self.neighbor_idx is None
                ) or not self.static:
                    pyramid = Delaunay(
                        swarm.position, qhull_options="QJ0.001 Qbb Qc Qx"
                    )
                    indices, index_pointer = pyramid.vertex_neighbor_vertices
                    # Insert all the neighbors for each particle in the idx array
                    self.neighbor_idx = np.array(
                        [
                            index_pointer[indices[i] : indices[i + 1]]
                            for i in range(swarm.n_particles)
                        ]
                    )

                idx_min = np.array(
                    [
                        swarm.pbest_cost[self.neighbor_idx[i]].argmin()
                        for i in range(len(self.neighbor_idx))
                    ]
                )
                best_neighbor = np.array(
                    [
                        self.neighbor_idx[i][idx_min[i]]
                        for i in range(len(self.neighbor_idx))
                    ]
                ).astype(int)

                # Obtain best cost and position
                best_cost = np.min(swarm.pbest_cost[best_neighbor])
                best_pos = swarm.pbest_pos[best_neighbor]
        except AttributeError:
            self.rep.logger.exception(
                "Please pass a Swarm class. You passed {}".format(type(swarm))
            )
            raise
        else:
            return (best_pos, best_cost)

    def compute_velocity(
        self,
        swarm,
        clamp=None,
        vh=VelocityHandler(strategy="unmodified"),
        bounds=None,
    ):
        """Compute the velocity matrix

        This method updates the velocity matrix using the best and current
        positions of the swarm. The velocity matrix is computed using the
        cognitive and social terms of the swarm.

        A sample usage can be seen with the following:

        .. code-block :: python

            import pyswarms.backend as P
            from pyswarms.backend.swarm import Swarm
            from pyswarms.backend.handlers import VelocityHandler
            from pyswarms.backend.topology import Pyramid

            my_swarm = P.create_swarm(n_particles, dimensions)
            my_topology = Pyramid(static=False)
            my_vh = VelocityHandler(strategy="zero")

            for i in range(iters):
                # Inside the for-loop
                my_swarm.velocity = my_topology.update_velocity(my_swarm, clamp, my_vh,
                bounds=bounds)

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        clamp : tuple of floats (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        vh : pyswarms.backend.handlers.VelocityHandler
            a VelocityHandler instance
        bounds : tuple of :code:`np.ndarray` or list (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum bound while
            the second entry is the maximum bound. Each array must be of shape
            :code:`(dimensions,)`.

        Returns
        -------
        numpy.ndarray
            Updated velocity matrix
        """
        return ops.compute_velocity(swarm, clamp, vh, bounds=bounds)

    def compute_position(
        self, swarm, bounds=None, bh=BoundaryHandler(strategy="periodic")
    ):
        """Update the position matrix

        This method updates the position matrix given the current position and
        the velocity. If bounded, it waives updating the position.

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        bounds : tuple of :code:`np.ndarray` or list (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum bound while
            the second entry is the maximum bound. Each array must be of shape
            :code:`(dimensions,)`.
        bh : a BoundaryHandler instance

        Returns
        -------
        numpy.ndarray
            New position-matrix
        """
        return ops.compute_position(swarm, bounds, bh)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import standard library
from collections import namedtuple

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.utils.functions import single_obj as fx


def test_ackley_output(common_minima):
    """Tests ackley function output."""
    assert np.isclose(fx.ackley(common_minima), np.zeros(3)).all()


def test_beale_output(common_minima2):
    """Tests beale function output."""
    assert np.isclose(fx.beale([3, 0.5] * common_minima2), np.zeros(3)).all()


def test_booth_output(common_minima2):
    """Test booth function output."""
    assert np.isclose(fx.booth([1, 3] * common_minima2), np.zeros(3)).all()


def test_bukin6_output(common_minima2):
    """Test bukin function output."""
    assert np.isclose(fx.bukin6([-10, 1] * common_minima2), np.zeros(3)).all()


@pytest.mark.parametrize(
    "x",
    [
        np.array(
            [
                [1.34941, -1.34941],
                [1.34941, 1.34941],
                [-1.34941, 1.34941],
                [-1.34941, -1.34941],
            ]
        )
    ],
)
@pytest.mark.parametrize(
    "minima", [np.array([-2.06261, -2.06261, -2.06261, -2.06261])]
)
def test_crossintray_output(x, minima):
    """Tests crossintray function output."""
    assert np.isclose(fx.crossintray(x), minima).all()


def test_easom_output(common_minima2):
    """Tests easom function output."""
    assert np.isclose(
        fx.easom([np.pi, np.pi] * common_minima2), (-1 * np.ones(3))
    ).all()


def test_eggholder_output(common_minima2):
    """Tests eggholder function output."""
    assert np.isclose(
        fx.eggholder([512, 404.3219] * common_minima2),
        (-959.6407 * np.ones(3)),
    ).all()


def test_goldstein_output(common_minima2):
    """Tests goldstein-price function output."""
    assert np.isclose(
        fx.goldstein([0, -1] * common_minima2), (3 * np.ones(3))
    ).all()


@pytest.mark.parametrize(
    "x",
    [
        np.array(
            [
                [3.0, 2.0],
                [-2.805118, 3.131312],
                [-3.779310, -3.283186],
                [3.584428, -1.848126],
            ]
        )
    ],
)
def test_himmelblau_output(x):
    """Tests himmelblau function output."""
    assert np.isclose(fx.himmelblau(x), np.zeros(4)).all()


@pytest.mark.parametrize(
    "x",
    [
        np.array(
            [
                [8.05502, 9.66459],
                [-8.05502, 9.66459],
                [8.05502, -9.66459],
                [-8.05502, -9.66459],
            ]
        )
    ],
)
@pytest.mark.parametrize(
    "minima", [np.array([-19.2085, -19.2085, -19.2085, -19.2085])]
)
def test_holdertable_output(x, minima):
    """Tests holdertable function output."""
    assert np.isclose(fx.holdertable(x), minima).all()


def test_levi_output(common_minima2):
    """Test levi function output."""
    assert np.isclose(fx.levi(common_minima2), np.zeros(3)).all()


def test_matyas_output(common_minima):
    """Test matyas function output."""
    assert np.isclose(fx.matyas(common_minima), np.zeros(3)).all()


def test_rastrigin_output(common_minima):
    """Tests rastrigin function output."""
    assert np.array_equal(fx.rastrigin(common_minima), np.zeros(3))


def test_rosenbrock_output(common_minima2):
    """Tests rosenbrock function output."""
    assert np.array_equal(
        fx.rosenbrock(common_minima2).all(), np.zeros(3).all()
    )


def test_schaffer2_output(common_minima):
    """Test schaffer2 function output."""
    assert np.isclose(fx.schaffer2(common_minima), np.zeros(3)).all()


def test_sphere_output(common_minima):
    """Tests sphere function output."""
    assert np.array_equal(fx.sphere(common_minima), np.zeros((3,)))


def test_threehump_output(common_minima):
    """Tests threehump function output."""
    assert np.array_equal(fx.threehump(common_minima), np.zeros(3))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.backend.topology import Star

from .abc_test_topology import ABCTestTopology

np.random.seed(4135157)


class TestStarTopology(ABCTestTopology):
    @pytest.fixture
    def topology(self):
        return Star

    @pytest.fixture
    def options(self):
        return {}

    def test_compute_gbest_return_values(self, swarm, options, topology):
        """Test if compute_gbest() gives the expected return values"""
        topo = topology()
        expected_cost = 1.0002528364353296
        expected_pos = np.array(
            [9.90438476e-01, 2.50379538e-03, 1.87405987e-05]
        )
        pos, cost = topo.compute_gbest(swarm, **options)
        assert cost == pytest.approx(expected_cost)
        assert pos == pytest.approx(expected_pos)



"""
The :code:`pyswarms.backend.topology` contains various topologies. They dictate
the behavior of the particles and implement three methods:
    - compute_best_particle(): gets the position and cost of the best particle   in the swarm
    - update_velocity(): updates the velocity-matrix depending on the topology.
    - update_position(): updates the position-matrix depending on the topology.
"""

from .base import Topology
from .star import Star
from .ring import Ring
from .pyramid import Pyramid
from .random import Random
from .von_neumann import VonNeumann


__all__ = ["Topology", "Star", "Ring", "Pyramid", "Random", "VonNeumann"]



# -*- coding: utf-8 -*-

"""
Swarm Generation Backend

This module abstracts how a swarm is generated. You can see its
implementation in our base classes. In addition, you can use all the methods
here to dictate how a swarm is initialized for your custom PSO.

"""

# Import standard library
import logging

# Import modules
import numpy as np

from ..utils.reporter import Reporter
from .swarms import Swarm

rep = Reporter(logger=logging.getLogger(__name__))


def generate_swarm(
    n_particles, dimensions, bounds=None, center=1.00, init_pos=None
):
    """Generate a swarm

    Parameters
    ----------
    n_particles : int
        number of particles to be generated in the swarm.
    dimensions: int
        number of dimensions to be generated in the swarm
    bounds : tuple of numpy.ndarray or list, optional
        a tuple of size 2 where the first entry is the minimum bound while
        the second entry is the maximum bound. Each array must be of shape
        :code:`(dimensions,)`. Default is :code:`None`
    center : numpy.ndarray or float, optional
        controls the mean or center whenever the swarm is generated randomly.
        Default is :code:`1`
    init_pos : numpy.ndarray, optional
        option to explicitly set the particles' initial positions. Set to
        :code:`None` if you wish to generate the particles randomly.
        Default is :code:`None`.

    Returns
    -------
    numpy.ndarray
        swarm matrix of shape (n_particles, n_dimensions)

    Raises
    ------
    ValueError
        When the shapes and values of bounds, dimensions, and init_pos
        are inconsistent.
    TypeError
        When the argument passed to bounds is not an iterable.
    """
    try:
        if (init_pos is not None) and (bounds is None):
            pos = init_pos
        elif (init_pos is not None) and (bounds is not None):
            if not (
                np.all(bounds[0] <= init_pos) and np.all(init_pos <= bounds[1])
            ):
                raise ValueError("User-defined init_pos is out of bounds.")
            pos = init_pos
        elif (init_pos is None) and (bounds is None):
            pos = center * np.random.uniform(
                low=0.0, high=1.0, size=(n_particles, dimensions)
            )
        else:
            lb, ub = bounds
            min_bounds = np.repeat(
                np.array(lb)[np.newaxis, :], n_particles, axis=0
            )
            max_bounds = np.repeat(
                np.array(ub)[np.newaxis, :], n_particles, axis=0
            )
            pos = center * np.random.uniform(
                low=min_bounds, high=max_bounds, size=(n_particles, dimensions)
            )
    except ValueError:
        msg = "Bounds and/or init_pos should be of size ({},)"
        rep.logger.exception(msg.format(dimensions))
        raise
    except TypeError:
        msg = "generate_swarm() takes an int for n_particles and dimensions and an array for bounds"
        rep.logger.exception(msg)
        raise
    else:
        return pos


def generate_discrete_swarm(
    n_particles, dimensions, binary=False, init_pos=None
):
    """Generate a discrete swarm

    Parameters
    ----------
    n_particles : int
        number of particles to be generated in the swarm.
    dimensions: int
        number of dimensions to be generated in the swarm.
    binary : bool
        generate a binary matrix. Default is :code:`False`
    init_pos : numpy.ndarray, optional
        option to explicitly set the particles' initial positions. Set to
        :code:`None` if you wish to generate the particles randomly.
        Default is :code:`None`

    Returns
    -------
    numpy.ndarray
        swarm matrix of shape (n_particles, n_dimensions)

    Raises
    ------
    ValueError
        When init_pos during binary=True does not contain two unique values.
    TypeError
        When the argument passed to n_particles or dimensions is incorrect.
    """
    try:
        if (init_pos is not None) and binary:
            if not len(np.unique(init_pos)) <= 2:
                raise ValueError("User-defined init_pos is not binary!")
            # init_pos maybe ones
            pos = init_pos
        elif (init_pos is not None) and not binary:
            pos = init_pos
        elif (init_pos is None) and binary:
            pos = np.random.randint(2, size=(n_particles, dimensions))
        else:
            pos = np.random.random_sample(
                size=(n_particles, dimensions)
            ).argsort(axis=1)
    except ValueError:
        rep.logger.exception("Please check the size and value of dimensions")
        raise
    except TypeError:
        msg = "generate_discrete_swarm() takes an int for n_particles and dimensions"
        rep.logger.exception(msg)
        raise
    else:
        return pos


def generate_velocity(n_particles, dimensions, clamp=None):
    """Initialize a velocity vector

    Parameters
    ----------
    n_particles : int
        number of particles to be generated in the swarm.
    dimensions: int
        number of dimensions to be generated in the swarm.
    clamp : tuple of floats, optional
        a tuple of size 2 where the first entry is the minimum velocity
        and the second entry is the maximum velocity. It
        sets the limits for velocity clamping. Default is :code:`None`

    Returns
    -------
    numpy.ndarray
        velocity matrix of shape (n_particles, dimensions)
    """
    try:
        min_velocity, max_velocity = (0, 1) if clamp is None else clamp
        velocity = (max_velocity - min_velocity) * np.random.random_sample(
            size=(n_particles, dimensions)
        ) + min_velocity
    except ValueError:
        msg = "Please check clamp shape: {} != {}"
        rep.logger.exception(msg.format(len(clamp), dimensions))
        raise
    except TypeError:
        msg = "generate_velocity() takes an int for n_particles and dimensions and an array for clamp"
        rep.logger.exception(msg)
        raise
    else:
        return velocity


def create_swarm(
    n_particles,
    dimensions,
    discrete=False,
    binary=False,
    options={},
    bounds=None,
    center=1.0,
    init_pos=None,
    clamp=None,
):
    """Abstract the generate_swarm() and generate_velocity() methods

    Parameters
    ----------
    n_particles : int
        number of particles to be generated in the swarm.
    dimensions: int
        number of dimensions to be generated in the swarm
    discrete : bool
        Creates a discrete swarm. Default is `False`
    options : dict, optional
        Swarm options, for example, c1, c2, etc.
    binary : bool
        generate a binary matrix, Default is `False`
    bounds : tuple of np.ndarray or list
        a tuple of size 2 where the first entry is the minimum bound while
        the second entry is the maximum bound. Each array must be of shape
        :code:`(dimensions,)`. Default is `None`
    center : numpy.ndarray, optional
        a list of initial positions for generating the swarm. Default is `1`
    init_pos : numpy.ndarray, optional
        option to explicitly set the particles' initial positions. Set to
        :code:`None` if you wish to generate the particles randomly.
    clamp : tuple of floats, optional
        a tuple of size 2 where the first entry is the minimum velocity
        and the second entry is the maximum velocity. It
        sets the limits for velocity clamping.

    Returns
    -------
    pyswarms.backend.swarms.Swarm
        a Swarm class
    """
    if discrete:
        position = generate_discrete_swarm(
            n_particles, dimensions, binary=binary, init_pos=init_pos
        )
    else:
        position = generate_swarm(
            n_particles,
            dimensions,
            bounds=bounds,
            center=center,
            init_pos=init_pos,
        )

    velocity = generate_velocity(n_particles, dimensions, clamp=clamp)
    return Swarm(position, velocity, options=options)



# Import modules
import numpy as np
import pytest

# Import from pyswarms
# Import from package
from pyswarms.utils.decorators import cost


@pytest.mark.parametrize("objective_func", [np.sum, np.prod])
def test_cost_decorator(objective_func, particles):
    """Test if cost decorator returns the same shape and value as undecorated function"""
    n_particles = particles.shape[0]

    def cost_func_without_decorator(x):
        n_particles_in_func = x.shape[0]
        cost = np.array(
            [objective_func(x[i]) for i in range(n_particles_in_func)]
        )
        return cost

    @cost
    def cost_func_with_decorator(x):
        cost = objective_func(x)
        return cost

    undecorated = cost_func_without_decorator(particles)
    decorated = cost_func_with_decorator(particles)

    assert np.array_equal(decorated, undecorated)
    assert decorated.shape == (n_particles,)


def test_decorator_invalid_cost_func(particles):
    """Test if ValueError is raised whenever an invalid cost function is passed"""

    def objective_func(x):
        """Returns a numpy.ndarray instead of int or float"""
        return np.array([1, 3])

    @cost
    def cost_func_with_wrong_output_shape_decorated(x):
        cost = objective_func(x)
        return cost

    with pytest.raises(ValueError):
        cost_func_with_wrong_output_shape_decorated(particles)



# -*- coding: utf-8 -*-

r"""
Base class for single-objective discrete Particle Swarm Optimization
implementations.

All methods here are abstract and raises a :code:`NotImplementedError`
when not used. When defining your own swarm implementation,
create another class,

    >>> class MySwarm(DiscreteSwarmOptimizer):
    >>>     def __init__(self):
    >>>        super(MySwarm, self).__init__()

and define all the necessary methods needed.

As a guide, check the discrete PSO implementations in this package.

.. note:: Regarding :code:`options`, it is highly recommended to
    include parameters used in position and velocity updates as
    keyword arguments. For parameters that affect the topology of
    the swarm, it may be much better to have them as positional
    arguments.

See Also
--------
:mod:`pyswarms.discrete.binary`: binary PSO implementation

"""

# Import standard library
import abc
from collections import namedtuple

# Import modules
import numpy as np

# Import from package
from ..backend import create_swarm


class DiscreteSwarmOptimizer(abc.ABC):
    def __init__(
        self,
        n_particles,
        dimensions,
        binary,
        options,
        velocity_clamp=None,
        init_pos=None,
        ftol=-np.inf,
    ):
        """Initialize the swarm.

        Creates a :code:`numpy.ndarray` of positions depending on the
        number of particles needed and the number of dimensions.
        The initial positions of the particles depends on the argument
        :code:`binary`, which governs if a binary matrix will be produced.

        Attributes
        ----------
        n_particles : int
            number of particles in the swarm.
        dimensions : int
            number of dimensions in the space.
        binary : boolean
            a trigger to generate a binary matrix for the swarm's
            initial positions. When passed with a :code:`False` value,
            random integers from 0 to :code:`dimensions` are generated.
        options : dict with keys :code:`{'c1', 'c2', 'w'}`
            a dictionary containing the parameters for the specific
            optimization technique
                * c1 : float
                    cognitive parameter
                * c2 : float
                    social parameter
                * w : float
                    inertia parameter
        velocity_clamp : tuple, optional
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        options: dict
            a dictionary containing the parameters for a specific
            optimization technique
        """
        # Initialize primary swarm attributes
        self.n_particles = n_particles
        self.dimensions = dimensions
        self.binary = binary
        self.velocity_clamp = velocity_clamp
        self.swarm_size = (n_particles, dimensions)
        self.options = options
        self.init_pos = init_pos
        self.ftol = ftol
        # Initialize named tuple for populating the history list
        self.ToHistory = namedtuple(
            "ToHistory",
            [
                "best_cost",
                "mean_pbest_cost",
                "mean_neighbor_cost",
                "position",
                "velocity",
            ],
        )

        # Initialize resettable attributes
        self.reset()

    def _populate_history(self, hist):
        """Populate all history lists

        The :code:`cost_history`, :code:`mean_pbest_history`, and
        :code:`neighborhood_best` is expected to have a shape of
        :code:`(iters,)`,on the other hand, the :code:`pos_history`
        and :code:`velocity_history` are expected to have a shape of
        :code:`(iters, n_particles, dimensions)`

        Parameters
        ----------
        hist : collections.namedtuple
            Must be of the same type as self.ToHistory
        """
        self.cost_history.append(hist.best_cost)
        self.mean_pbest_history.append(hist.mean_pbest_cost)
        self.mean_neighbor_history.append(hist.mean_neighbor_cost)
        self.pos_history.append(hist.position)
        self.velocity_history.append(hist.velocity)

    @abc.abstractmethod
    def optimize(self, objective_func, iters, n_processes=None, **kwargs):
        """Optimize the swarm for a number of iterations

        Performs the optimization to evaluate the objective
        function :code:`objective_func` for a number of iterations
        :code:`iter.`

        Parameters
        ----------
        objective_func : callable
            objective function to be evaluated
        iters : int
            number of iterations
        n_processes : int
            number of processes to use for parallel particle evaluation
            Default is None with no parallelization
        kwargs : dict
            arguments for objective function

        Raises
        ------
        NotImplementedError
            When this method is not implemented.
        """
        raise NotImplementedError("SwarmBase::optimize()")

    def reset(self):
        """Reset the attributes of the optimizer

        All variables/atributes that will be re-initialized when this
        method is defined here. Note that this method
        can be called twice: (1) during initialization, and (2) when
        this is called from an instance.

        It is good practice to keep the number of resettable
        attributes at a minimum. This is to prevent spamming the same
        object instance with various swarm definitions.

        Normally, swarm definitions are as atomic as possible, where
        each type of swarm is contained in its own instance. Thus, the
        following attributes are the only ones recommended to be
        resettable:

        * Swarm position matrix (self.pos)
        * Velocity matrix (self.pos)
        * Best scores and positions (gbest_cost, gbest_pos, etc.)

        Otherwise, consider using positional arguments.
        """
        # Initialize history lists
        self.cost_history = []
        self.mean_pbest_history = []
        self.mean_neighbor_history = []
        self.pos_history = []
        self.velocity_history = []

        # Initialize the swarm
        self.swarm = create_swarm(
            n_particles=self.n_particles,
            dimensions=self.dimensions,
            discrete=True,
            init_pos=self.init_pos,
            binary=self.binary,
            clamp=self.velocity_clamp,
            options=self.options,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pyswarms` package."""

# Import standard library
from collections import namedtuple

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.utils.functions import single_obj as fx


def test_beale_dim_fail(outdim):
    """Test beale dim exception"""
    with pytest.raises(IndexError):
        fx.beale(outdim)


def test_booth_dim_fail(outdim):
    """Test booth dim exception"""
    with pytest.raises(IndexError):
        fx.booth(outdim)


def test_bukin6_dim_fail(outdim):
    """Test bukin6 dim exception"""
    with pytest.raises(IndexError):
        fx.bukin6(outdim)


def test_crossintray_dim_fail(outdim):
    """Test crossintray dim exception"""
    with pytest.raises(IndexError):
        fx.crossintray(outdim)


def test_easom_dim_fail(outdim):
    """Test easom dim exception"""
    with pytest.raises(IndexError):
        fx.easom(outdim)


def test_goldstein_dim_fail(outdim):
    """Test goldstein dim exception"""
    with pytest.raises(IndexError):
        fx.goldstein(outdim)


def test_eggholder_dim_fail(outdim):
    """Test eggholder dim exception"""
    with pytest.raises(IndexError):
        fx.eggholder(outdim)


def test_himmelblau_dim_fail(outdim):
    """Test himmelblau dim exception"""
    with pytest.raises(IndexError):
        fx.himmelblau(outdim)


def test_holdertable_dim_fail(outdim):
    """Test holdertable dim exception"""
    with pytest.raises(IndexError):
        fx.holdertable(outdim)


def test_levi_dim_fail(outdim):
    """Test levi dim exception"""
    with pytest.raises(IndexError):
        fx.levi(outdim)


def test_matyas_dim_fail(outdim):
    """Test matyas dim exception"""
    with pytest.raises(IndexError):
        fx.matyas(outdim)


def test_schaffer2_dim_fail(outdim):
    """Test schaffer2 dim exception"""
    with pytest.raises(IndexError):
        fx.schaffer2(outdim)


def test_threehump_dim_fail(outdim):
    """Test threehump dim exception"""
    with pytest.raises(IndexError):
        fx.threehump(outdim)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Fixtures for tests"""

# Import modules
import numpy as np
import pytest

# Import from pyswarms
# Import from package
from pyswarms.backend.swarms import Swarm


@pytest.fixture
def swarm():
    """A contrived instance of the Swarm class at a certain timestep"""
    attrs_at_t = {
        "position": np.array([[5, 5, 5], [3, 3, 3], [1, 1, 1]]),
        "velocity": np.array([[1, 1, 1], [1, 1, 1], [1, 1, 1]]),
        "current_cost": np.array([2, 2, 2]),
        "pbest_cost": np.array([1, 2, 3]),
        "pbest_pos": np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]]),
        "best_cost": 1,
        "best_pos": np.array([1, 1, 1]),
        "options": {"c1": 0.5, "c2": 1, "w": 2},
    }
    return Swarm(**attrs_at_t)

@pytest.fixture
def bounds():
    bounds_ = (np.array([2, 3, 1]), np.array([4, 7, 8]))
    return bounds_

@pytest.fixture
def clamp():
    clamp_ = (np.array([2, 3, 1]), np.array([4, 7, 8]))
    return clamp_

@pytest.fixture
def positions_inbound():
    pos_ = np.array([[3.3, 4.4, 2.3],
                     [3.7, 5.2, 7.0],
                     [2.5, 6.8, 2.3],
                     [2.1, 6.9, 4.7],
                     [2.7, 3.2, 3.5],
                     [2.5, 5.1, 1.2]
                    ])
    return pos_

@pytest.fixture
def positions_out_of_bound():
    pos_ = np.array([[5.3, 4.4, 2.3],
                     [3.7, 9.2, 7.0],
                     [8.5, 0.8, 2.3],
                     [2.1, 6.9, 0.7],
                     [2.7, 9.2, 3.5],
                     [1.5, 5.1, 9.2]
                    ])
    return pos_

@pytest.fixture
def velocities_inbound():
    pos_ = np.array([[3.3, 4.4, 2.3],
                     [3.7, 5.2, 7.0],
                     [2.5, 6.8, 2.3],
                     [2.1, 6.9, 4.7],
                     [2.7, 3.2, 3.5],
                     [2.5, 5.1, 1.2]
                    ])
    return pos_

@pytest.fixture
def velocities_out_of_bound():
    pos_ = np.array([[5.3, 4.4, 2.3],
                     [3.7, 9.2, 7.0],
                     [8.5, 0.8, 2.3],
                     [2.1, 6.9, 0.7],
                     [2.7, 9.2, 3.5],
                     [1.5, 5.1, 9.2]
                    ])
    return pos_



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.backend.topology import Pyramid
from .abc_test_topology import ABCTestTopology

np.random.seed(4135157)


class TestPyramidTopology(ABCTestTopology):
    @pytest.fixture
    def topology(self):
        return Pyramid

    @pytest.fixture
    def options(self):
        return {}

    @pytest.mark.parametrize("static", [True, False])
    def test_compute_gbest_return_values(
        self, swarm, topology, options, static
    ):
        """Test if compute_gbest() gives the expected return values"""
        topo = topology(static=static)
        expected_cost = 1.0002528364353296
        expected_pos = np.array([9.90438476e-01, 2.50379538e-03, 1.87405987e-05])
        pos, cost = topo.compute_gbest(swarm, **options)
        assert cost == pytest.approx(expected_cost)
        assert pos[np.argmin(cost)] == pytest.approx(expected_pos)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Fixtures for tests

isort:skip_file
"""

# Import standard library
import os

# Import modules
import matplotlib as mpl
import numpy as np
import pytest

if os.environ.get("DISPLAY", "") == "":
    mpl.use("Agg")

# Import from pyswarms
from pyswarms.single import GlobalBestPSO
from pyswarms.utils.functions.single_obj import sphere
from pyswarms.utils.plotters.formatters import Mesher


@pytest.fixture
def trained_optimizer():
    """Returns a trained optimizer instance with 100 iterations"""
    options = {"c1": 0.5, "c2": 0.3, "w": 0.9}
    optimizer = GlobalBestPSO(n_particles=10, dimensions=2, options=options)
    optimizer.optimize(sphere, iters=100)
    return optimizer


@pytest.fixture
def pos_history():
    """Returns a list containing a swarms' position history"""
    return np.random.uniform(size=(10, 5, 2))


@pytest.fixture
def mesher():
    """A Mesher instance with sphere function and delta=0.1"""
    return Mesher(func=sphere, delta=0.1)



# -*- coding: utf-8 -*-
"""Base class for hyperparameter optimization search functions"""

# Import from __future__
from __future__ import absolute_import, print_function, with_statement

# Import standard library
import operator as op


class SearchBase(object):
    def assertions(self):
        """Assertion method to check :code:`optimizer` input

        Raises
        ------
        TypeError
            When :code:`optimizer` does not have an `'optimize'` attribute.
        """
        # Check type of optimizer object
        if not hasattr(self.optimizer, "optimize"):
            raise TypeError(
                "Parameter `optimizer` must have an " "`'optimize'` attribute."
            )

    def __init__(
        self,
        optimizer,
        n_particles,
        dimensions,
        options,
        objective_func,
        iters,
        bounds=None,
        velocity_clamp=(0, 1),
    ):
        """Initialize the Search

        Attributes
        ----------
        optimizer: pyswarms.single
            either LocalBestPSO or GlobalBestPSO
        n_particles : int
            number of particles in the swarm.
        dimensions : int
            number of dimensions in the space.
        options : dict with keys :code:`{'c1', 'c2', 'w', 'k', 'p'}`
            a dictionary containing the parameters for the specific
            optimization technique
                * c1 : float
                    cognitive parameter
                * c2 : float
                    social parameter
                * w : float
                    inertia parameter
                * k : int
                    number of neighbors to be considered. Must be a
                    positive integer less than :code:`n_particles`
                * p: int {1,2}
                    the Minkowski p-norm to use. 1 is the
                    sum-of-absolute values (or L1 distance) while 2 is
                    the Euclidean (or L2) distance.
        objective_func: function
            objective function to be evaluated
        iters: int
            number of iterations
        bounds : tuple of np.ndarray, optional (default is None)
            a tuple of size 2 where the first entry is the minimum bound
            while the second entry is the maximum bound. Each array must
            be of shape :code:`(dimensions,)`.
        velocity_clamp : tuple (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        """

        # Assign attributes
        self.optimizer = optimizer
        self.n_particles = n_particles
        self.dims = dimensions
        self.options = options
        self.bounds = bounds
        self.vclamp = velocity_clamp
        self.objective_func = objective_func
        self.iters = iters
        # Invoke assertions
        self.assertions()

    def generate_score(self, options):
        """Generate score for optimizer's performance on objective function

        Parameters
        ----------

        options: dict
            a dict with the following keys: {'c1', 'c2', 'w', 'k', 'p'}
        """

        # Intialize optimizer
        f = self.optimizer(
            self.n_particles, self.dims, options, self.bounds, velocity_clamp=self.vclamp
        )

        # Return score
        return f.optimize(self.objective_func, self.iters)[0]

    def search(self, maximum=False):
        """Compare optimizer's objective function performance scores
        for all combinations of provided parameters

        Parameters
        ----------

        maximum: bool
            a bool defaulting to False, returning the minimum value for the
            objective function. If set to True, will return the maximum value
            for the objective function.
        """

        # Generate the grid of all hyperparameter value combinations
        grid = self.generate_grid()

        # Calculate scores for all hyperparameter combinations
        scores = [self.generate_score(i) for i in grid]

        # Default behavior
        idx, self.best_score = min(enumerate(scores), key=op.itemgetter(1))

        # Catches the maximum bool flag
        if maximum:
            idx, self.best_score = max(enumerate(scores), key=op.itemgetter(1))

        # Return optimum hyperparameter value property from grid using index
        self.best_options = op.itemgetter(idx)(grid)
        return self.best_score, self.best_options



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest
import numpy as np

# Import from pyswarms
from pyswarms.single import GlobalBestPSO
from pyswarms.utils.functions.single_obj import sphere

from .abc_test_optimizer import ABCTestOptimizer


class TestGlobalBestOptimizer(ABCTestOptimizer):
    @pytest.fixture
    def optimizer(self):
        return GlobalBestPSO

    @pytest.fixture
    def optimizer_history(self, options):
        opt = GlobalBestPSO(10, 2, options=options)
        opt.optimize(sphere, 1000)
        return opt

    @pytest.fixture
    def optimizer_reset(self, options):
        opt = GlobalBestPSO(10, 2, options=options)
        opt.optimize(sphere, 10)
        opt.reset()
        return opt

    def test_global_correct_pos(self, options):
        """ Test to check global optimiser returns the correct position corresponding to the best cost """
        opt = GlobalBestPSO(n_particles=10, dimensions=2, options=options)
        cost, pos = opt.optimize(sphere, iters=5)
        # find best pos from history
        min_cost_idx = np.argmin(opt.cost_history)
        min_pos_idx = np.argmin(sphere(opt.pos_history[min_cost_idx]))
        assert np.array_equal(opt.pos_history[min_cost_idx][min_pos_idx], pos)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import numpy as np
import pytest

# Import from pyswarms
import pyswarms.backend as P


class TestGenerateSwarm(object):
    """Test suite for generate_swarm() method"""

    @pytest.mark.parametrize(
        "bounds", [None, ([2, 2, 2], [5, 5, 5]), ([-1, -1, 0], [2, 2, 5])]
    )
    @pytest.mark.parametrize("center", [1, [3, 3, 3], [0.2, 0.2, 0.1]])
    def test_return_values(self, bounds, center):
        """Test if method returns expected values"""
        pos = P.generate_swarm(
            n_particles=2, dimensions=3, bounds=bounds, center=center
        )
        if bounds is None:
            min_bounds, max_bounds = (0.0, 1.00)
        else:
            min_bounds, max_bounds = bounds
        lower_bound = center * np.array(min_bounds)
        upper_bound = center * np.array(max_bounds)
        assert (pos <= upper_bound).all() and (pos >= lower_bound).all()

    def test_out_of_bounds(self):
        """Test if method raises ValueError when initialized with the wrong value"""
        bounds = ([1, 1, 1], [5, 5, 5])
        init_pos = np.array([[-2, 3, 3], [6, 8, 1]])
        with pytest.raises(ValueError):
            P.generate_swarm(
                n_particles=2, dimensions=3, bounds=bounds, init_pos=init_pos
            )

    @pytest.mark.parametrize("bounds", [0.1])
    def test_bounds_wrong_type(self, bounds):
        """Test if method raises TypeError when bounds is not an array"""
        with pytest.raises(TypeError):
            P.generate_swarm(n_particles=2, dimensions=3, bounds=bounds)

    @pytest.mark.parametrize(
        "bounds", [(1, 1, 1), ([1, 1, 1]), ([1, 1, 1], [2, 2])]
    )
    def test_bounds_wrong_size(self, bounds):
        """Test if method raises ValueError when bounds is of wrong shape"""
        with pytest.raises(ValueError):
            P.generate_swarm(n_particles=2, dimensions=3, bounds=bounds)


class TestDiscreteSwarm(object):
    """Test suite for generate_discrete_swarm() method"""

    @pytest.mark.parametrize("binary", [False, True])
    def test_generate_discrete_binary_swarm(self, binary):
        """Test if binary=True returns expected values"""
        dims = 3
        pos = P.generate_discrete_swarm(
            n_particles=2, dimensions=dims, binary=binary
        )
        if binary:
            assert len(np.unique(pos)) <= 2  # Might generate pure 0 or 1
        else:
            assert (np.max(pos, axis=1) == dims - 1).all()

    def test_not_binary_error_discrete_swarm(self):
        """Test if method raises ValueError given wrong init_pos val"""
        init_pos = [0, 1, 2]
        with pytest.raises(ValueError):
            P.generate_discrete_swarm(
                n_particles=2, dimensions=3, binary=True, init_pos=init_pos
            )

    @pytest.mark.parametrize(
        "init_pos", [None, np.array([[4, 2, 1], [1, 4, 6]])]
    )
    def test_generate_discrete_swarm(self, init_pos):
        """Test if init_pos actually sets the position properly"""
        dims = 3
        pos = P.generate_discrete_swarm(
            n_particles=2, dimensions=dims, init_pos=init_pos
        )
        if init_pos is None:
            assert (np.max(pos, axis=1) == dims - 1).all()
        else:
            assert np.equal(pos, init_pos).all()


class TestGenerateVelocity(object):
    """Test suite for generate_velocity()"""

    @pytest.mark.parametrize("clamp", [None, (0, 1), (2, 5), (1, 6)])
    def test_return_values(self, clamp):
        """Test if the method returns expected values"""
        min_clamp, max_clamp = (0, 1) if clamp is None else clamp
        velocity = P.generate_velocity(
            n_particles=2, dimensions=3, clamp=clamp
        )
        assert (velocity <= max_clamp).all() and (velocity >= min_clamp).all()

    @pytest.mark.parametrize("clamp", [(0, 2, 5), [1, 3, 5]])
    def test_invalid_clamp_value(self, clamp):
        """Test if the method raises a ValueError given invalid clamp size"""
        with pytest.raises(ValueError):
            P.generate_velocity(n_particles=2, dimensions=3, clamp=clamp)

    @pytest.mark.parametrize("clamp", [0, 1])
    def test_invalid_clamp_type(self, clamp):
        """Test if method raises a TypeError given invalid clamp type"""
        with pytest.raises(TypeError):
            P.generate_velocity(n_particles=2, dimensions=3, clamp=clamp)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import standard library
from collections import namedtuple

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.utils.functions import single_obj as fx


def test_ackley_output_size(common_minima, targetdim):
    """Tests ackley output size."""
    assert fx.ackley(common_minima).shape == targetdim


def test_beale_output_size(common_minima, targetdim):
    """Tests beale output size."""
    assert fx.beale(common_minima).shape == targetdim


def test_booth_output_size(common_minima, targetdim):
    """Test booth output size."""
    assert fx.booth(common_minima).shape == targetdim


def test_bukin6_output_size(common_minima2, targetdim):
    """Test bukin6 output size."""
    assert fx.bukin6([-10, 0] * common_minima2).shape == targetdim


def test_crossintray_output_size(common_minima2, targetdim):
    """Test crossintray output size."""
    assert fx.crossintray([-10, 0] * common_minima2).shape == targetdim


def test_easom_output_size(common_minima2, targetdim):
    """Test easom output size."""
    assert fx.easom([-10, 0] * common_minima2).shape == targetdim


def test_eggholder_output_size(common_minima2, targetdim):
    """Test eggholder output size."""
    assert fx.eggholder([-10, 0] * common_minima2).shape == targetdim


def test_goldstein_output_size(common_minima, targetdim):
    """Test goldstein output size."""
    assert fx.goldstein(common_minima).shape == targetdim


def test_himmelblau_output_size(common_minima, targetdim):
    """Test himmelblau output size."""
    assert fx.himmelblau(common_minima).shape == targetdim


def test_holdertable_output_size(common_minima, targetdim):
    """Test holdertable output size."""
    assert fx.holdertable(common_minima).shape == targetdim


def test_levi_output_size(common_minima, targetdim):
    """Test levi output size."""
    assert fx.levi(common_minima).shape == targetdim


def test_rastrigin_output_size(common_minima, targetdim):
    """Tests rastrigin output size."""
    assert fx.rastrigin(common_minima).shape == targetdim


def test_rosenbrock_output_size(common_minima, targetdim):
    """Tests rosenbrock output size."""
    assert fx.rosenbrock(common_minima).shape == targetdim


def test_schaffer2_output_size(common_minima, targetdim):
    """Test schaffer2 output size."""
    assert fx.schaffer2(common_minima).shape == targetdim


def test_sphere_output_size(common_minima, targetdim):
    """Tests sphere output size."""
    assert fx.sphere(common_minima).shape == targetdim


def test_threehump_output_size(common_minima, targetdim):
    """Test threehump output size."""
    assert fx.threehump(common_minima).shape == targetdim



# -*- coding: utf-8 -*-

"""
A Von Neumann Network Topology

This class implements a Von Neumann topology.
"""

# Import standard library
import logging

from ...utils.reporter import Reporter
from .ring import Ring


class VonNeumann(Ring):
    def __init__(self, static=None):
        # static = None is just an artifact to make the API consistent
        # Setting it will not change swarm behavior
        super(VonNeumann, self).__init__(static=True)
        self.rep = Reporter(logger=logging.getLogger(__name__))

    def compute_gbest(self, swarm, p, r, **kwargs):
        """Updates the global best using a neighborhood approach

        The Von Neumann topology inherits from the Ring topology and uses
        the same approach to calculate the global best. The number of
        neighbors is determined by the dimension and the range. This
        topology is always a :code:`static` topology.

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        r : int
            range of the Von Neumann topology
        p: int {1,2}
            the Minkowski p-norm to use. 1 is the
            sum-of-absolute values (or L1 distance) while 2 is
            the Euclidean (or L2) distance.

        Returns
        -------
        numpy.ndarray
            Best position of shape :code:`(n_dimensions, )`
        float
            Best cost
        """
        k = VonNeumann.delannoy(swarm.dimensions, r)
        return super(VonNeumann, self).compute_gbest(swarm, p, k)

    @staticmethod
    def delannoy(d, r):
        """Static helper method to compute Delannoy numbers

        This method computes the number of neighbours of a Von Neumann
        topology, i.e. a Delannoy number, dependent on the range and the
        dimension of the search space. The Delannoy numbers are computed
        recursively.

        Parameters
        ----------
        d : int
            dimension of the search space
        r : int
            range of the Von Neumann topology

        Returns
        -------
        int
            Delannoy number"""
        if d == 0 or r == 0:
            return 1
        else:
            del_number = (
                VonNeumann.delannoy(d - 1, r)
                + VonNeumann.delannoy(d - 1, r - 1)
                + VonNeumann.delannoy(d, r - 1)
            )
            return del_number



# -*- coding: utf-8 -*-
# Import standard library
import logging
import logging.config
import os
import pprint

# Import modules
import yaml
from tqdm import trange


class Reporter(object):
    """A Reporter object that abstracts various logging capabilities

    To set-up a Reporter, simply perform the following tasks:

    .. code-block:: python

        from pyswarms.utils import Reporter

        rep = Reporter()
        rep.log("Here's my message", lvl=logging.INFO)

    This will set-up a reporter with a default configuration that
    logs to a file, `report.log`, on the current working directory.
    You can change the log path by passing a string to the `log_path`
    parameter:

    .. code-block:: python

        from pyswarms.utils import Reporter

        rep = Reporter(log_path="/path/to/log/file.log")
        rep.log("Here's my message", lvl=logging.INFO)

    If you are working on a module and you have an existing logger,
    you can pass that logger instance during initialization:

    .. code-block:: python

        # mymodule.py
        from pyswarms.utils import Reporter

        # An existing logger in a module
        logger = logging.getLogger(__name__)
        rep = Reporter(logger=logger)

    Lastly, if you have your own logger configuration (YAML file),
    then simply pass that to the `config_path` parameter. This
    overrides the default configuration (including `log_path`):

    .. code-block:: python

        from pyswarms.utils import Reporter

        rep = Reporter(config_path="/path/to/config/file.yml")
        rep.log("Here's my message", lvl=logging.INFO)

    """

    def __init__(
        self, log_path=None, config_path=None, logger=None, printer=None
    ):
        """Initialize the reporter

        Attributes
        ----------
        log_path : str, optional
            Sets the default log path (overriden when :code:`path` is given to
            :code:`_setup_logger()`)
        config_path : str, optional
            Sets the configuration path for custom loggers
        logger : logging.Logger, optional
            The logger object. By default, it creates a new :code:`Logger`
            instance
        printer : pprint.PrettyPrinter, optional
            A printer object. By default, it creates a :code:`PrettyPrinter`
            instance with default values
        """
        self.logger = logger or logging.getLogger(__name__)
        self.printer = printer or pprint.PrettyPrinter()
        self.log_path = log_path or (os.getcwd() + "/report.log")
        self._bar_fmt = "{l_bar}{bar}|{n_fmt}/{total_fmt}{postfix}"
        self._env_key = "LOG_CFG"
        self._default_config = {
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {
                "standard": {
                    "format": "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
                }
            },
            "handlers": {
                "default": {
                    "level": "INFO",
                    "class": "logging.StreamHandler",
                    "formatter": "standard",
                },
                "file_default": {
                    "level": "INFO",
                    "formatter": "standard",
                    "class": "logging.handlers.RotatingFileHandler",
                    "filename": self.log_path,
                    "encoding": "utf8",
                    "maxBytes": 10485760,
                    "backupCount": 20,
                },
            },
            "loggers": {
                "": {
                    "handlers": ["default", "file_default"],
                    "level": "INFO",
                    "propagate": True,
                }
            },
        }
        self._setup_logger(config_path)

    def log(self, msg, lvl=logging.INFO, *args, **kwargs):
        """Log a message within a set level

        This method abstracts the logging.Logger.log() method. We use this
        method during major state changes, errors, or critical events during
        the optimization run.

        You can check logging levels on this `link`_. In essence, DEBUG is 10,
        INFO is 20, WARNING is 30, ERROR is 40, and CRITICAL is 50.

        .. link: https://docs.python.org/3/library/logging.html#logging-levels

        Parameters
        ----------
        msg : str
            Message to be logged
        lvl : int, optional
            Logging level. Default is `logging.INFO`
        """
        self.logger.log(lvl, msg, *args, **kwargs)

    def print(self, msg, verbosity, threshold=0):
        """Print a message into console

        This method can be called during non-system calls or minor state
        changes. In practice, we call this method when reporting the cost
        on a given timestep.

        Parameters
        ----------
        msg : str
            Message to be printed
        verbosity : int
            Verbosity parameter, prints message when it's greater than the
            threshold
        threshold : int, optional
            Threshold parameter, prints message when it's lesser than the
            verbosity. Default is `0`
        """
        if verbosity > threshold:
            self.printer.pprint(msg)
        else:
            pass

    def _setup_logger(self, path=None):
        """Set-up the logger with default values

        This method is called right after initializing the Reporter module.
        If no path is supplied, then it loads a default configuration.
        You can view the defaults via the Reporter._default_config attribute.


        Parameters
        ----------
        path : str, optional
            Path to a YAML configuration. If not supplied, uses
            a default config.
        """
        value = path or os.getenv(self._env_key, None)
        try:
            with open(value, "rt") as f:
                config = yaml.safe_load(f.read())
            logging.config.dictConfig(config)
        except (TypeError, FileNotFoundError):
            self._load_defaults()

    def _load_defaults(self):
        """Load default logging configuration"""
        logging.config.dictConfig(self._default_config)

    def pbar(self, iters, desc=None):
        """Create a tqdm iterable

        You can use this method to create progress bars. It uses a set
        of abstracted methods from tqdm:

        .. code-block:: python

            from pyswarms.utils import Reporter

            rep = Reporter()
            # Create a progress bar
            for i in rep.pbar(100, name="Optimizer")
                    pass

        Parameters
        ----------
        iters : int
            Maximum range passed to the tqdm instance
        desc : str, optional
            Name of the progress bar that will be displayed

        Returns
        -------
        :obj:`tqdm._tqdm.tqdm`
            A tqdm iterable
        """
        self.t = trange(iters, desc=desc, bar_format=self._bar_fmt)
        return self.t

    def hook(self, *args, **kwargs):
        """Set a hook on the progress bar

        Method for creating a postfix in tqdm. In practice we use this
        to report the best cost found during an iteration:

        .. code-block:: python

            from pyswarms.utils import Reporter

            rep = Reporter()
            # Create a progress bar
            for i in rep.pbar(100, name="Optimizer")
                    best_cost = compute()
                    rep.hook(best_cost=best_cost)
        """
        self.t.set_postfix(*args, **kwargs)



# -*- coding: utf-8 -*-

"""
Swarm Operation Backend

This module abstracts various operations in the swarm such as updating
the personal best, finding neighbors, etc. You can use these methods
to specify how the swarm will behave.
"""

# Import standard library
import logging

# Import modules
import numpy as np

from ..utils.reporter import Reporter
from .handlers import BoundaryHandler, VelocityHandler
from functools import partial


rep = Reporter(logger=logging.getLogger(__name__))


def compute_pbest(swarm):
    """Update the personal best score of a swarm instance

    You can use this method to update your personal best positions.

    .. code-block:: python

        import pyswarms.backend as P
        from pyswarms.backend.swarms import Swarm

        my_swarm = P.create_swarm(n_particles, dimensions)

        # Inside the for-loop...
        for i in range(iters):
            # It updates the swarm internally
            my_swarm.pbest_pos, my_swarm.pbest_cost = P.update_pbest(my_swarm)

    It updates your :code:`current_pbest` with the personal bests acquired by
    comparing the (1) cost of the current positions and the (2) personal
    bests your swarm has attained.

    If the cost of the current position is less than the cost of the personal
    best, then the current position replaces the previous personal best
    position.

    Parameters
    ----------
    swarm : pyswarms.backend.swarm.Swarm
        a Swarm instance

    Returns
    -------
    numpy.ndarray
        New personal best positions of shape :code:`(n_particles, n_dimensions)`
    numpy.ndarray
        New personal best costs of shape :code:`(n_particles,)`
    """
    try:
        # Infer dimensions from positions
        dimensions = swarm.dimensions
        # Create a 1-D and 2-D mask based from comparisons
        mask_cost = swarm.current_cost < swarm.pbest_cost
        mask_pos = np.repeat(mask_cost[:, np.newaxis], dimensions, axis=1)
        # Apply masks
        new_pbest_pos = np.where(~mask_pos, swarm.pbest_pos, swarm.position)
        new_pbest_cost = np.where(
            ~mask_cost, swarm.pbest_cost, swarm.current_cost
        )
    except AttributeError:
        rep.logger.exception(
            "Please pass a Swarm class. You passed {}".format(type(swarm))
        )
        raise
    else:
        return (new_pbest_pos, new_pbest_cost)


def compute_velocity(swarm, clamp, vh, bounds=None):
    """Update the velocity matrix

    This method updates the velocity matrix using the best and current
    positions of the swarm. The velocity matrix is computed using the
    cognitive and social terms of the swarm. The velocity is handled
    by a :code:`VelocityHandler`.

    A sample usage can be seen with the following:

    .. code-block :: python

        import pyswarms.backend as P
        from pyswarms.swarms.backend import Swarm, VelocityHandler

        my_swarm = P.create_swarm(n_particles, dimensions)
        my_vh = VelocityHandler(strategy="invert")

        for i in range(iters):
            # Inside the for-loop
            my_swarm.velocity = compute_velocity(my_swarm, clamp, my_vh, bounds)

    Parameters
    ----------
    swarm : pyswarms.backend.swarms.Swarm
        a Swarm instance
    clamp : tuple of floats, optional
        a tuple of size 2 where the first entry is the minimum velocity
        and the second entry is the maximum velocity. It
        sets the limits for velocity clamping.
    vh : pyswarms.backend.handlers.VelocityHandler
        a VelocityHandler object with a specified handling strategy.
        For further information see :mod:`pyswarms.backend.handlers`.
    bounds : tuple of numpy.ndarray or list, optional
        a tuple of size 2 where the first entry is the minimum bound while
        the second entry is the maximum bound. Each array must be of shape
        :code:`(dimensions,)`.

    Returns
    -------
    numpy.ndarray
        Updated velocity matrix
    """
    try:
        # Prepare parameters
        swarm_size = swarm.position.shape
        c1 = swarm.options["c1"]
        c2 = swarm.options["c2"]
        w = swarm.options["w"]
        # Compute for cognitive and social terms
        cognitive = (
            c1
            * np.random.uniform(0, 1, swarm_size)
            * (swarm.pbest_pos - swarm.position)
        )
        social = (
            c2
            * np.random.uniform(0, 1, swarm_size)
            * (swarm.best_pos - swarm.position)
        )
        # Compute temp velocity (subject to clamping if possible)
        temp_velocity = (w * swarm.velocity) + cognitive + social
        updated_velocity = vh(
            temp_velocity, clamp, position=swarm.position, bounds=bounds
        )

    except AttributeError:
        rep.logger.exception(
            "Please pass a Swarm class. You passed {}".format(type(swarm))
        )
        raise
    except KeyError:
        rep.logger.exception("Missing keyword in swarm.options")
        raise
    else:
        return updated_velocity


def compute_position(swarm, bounds, bh):
    """Update the position matrix

    This method updates the position matrix given the current position and the
    velocity. If bounded, the positions are handled by a
    :code:`BoundaryHandler` instance

    .. code-block :: python

        import pyswarms.backend as P
        from pyswarms.swarms.backend import Swarm, VelocityHandler

        my_swarm = P.create_swarm(n_particles, dimensions)
        my_bh = BoundaryHandler(strategy="intermediate")

        for i in range(iters):
            # Inside the for-loop
            my_swarm.position = compute_position(my_swarm, bounds, my_bh)

    Parameters
    ----------
    swarm : pyswarms.backend.swarms.Swarm
        a Swarm instance
    bounds : tuple of numpy.ndarray or list, optional
        a tuple of size 2 where the first entry is the minimum bound while
        the second entry is the maximum bound. Each array must be of shape
        :code:`(dimensions,)`.
    bh : pyswarms.backend.handlers.BoundaryHandler
        a BoundaryHandler object with a specified handling strategy
        For further information see :mod:`pyswarms.backend.handlers`.

    Returns
    -------
    numpy.ndarray
        New position-matrix
    """
    try:
        temp_position = swarm.position.copy()
        temp_position += swarm.velocity

        if bounds is not None:
            temp_position = bh(temp_position, bounds)

        position = temp_position
    except AttributeError:
        rep.logger.exception(
            "Please pass a Swarm class. You passed {}".format(type(swarm))
        )
        raise
    else:
        return position


def compute_objective_function(swarm, objective_func, pool=None, **kwargs):
    """Evaluate particles using the objective function

    This method evaluates each particle in the swarm according to the objective
    function passed.

    If a pool is passed, then the evaluation of the particles is done in
    parallel using multiple processes.

    Parameters
    ----------
    swarm : pyswarms.backend.swarms.Swarm
        a Swarm instance
    objective_func : function
        objective function to be evaluated
    pool: multiprocessing.Pool
        multiprocessing.Pool to be used for parallel particle evaluation
    kwargs : dict
        arguments for the objective function

    Returns
    -------
    numpy.ndarray
        Cost-matrix for the given swarm
    """
    if pool is None:
        return objective_func(swarm.position, **kwargs)
    else:
        results = pool.map(
            partial(objective_func, **kwargs),
            np.array_split(swarm.position, pool._processes),
        )
        return np.concatenate(results)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.backend.topology import Random
from .abc_test_topology import ABCTestTopology

np.random.seed(4135157)


class TestRandomTopology(ABCTestTopology):
    @pytest.fixture
    def topology(self):
        return Random

    @pytest.fixture(params=[1, 2, 3])
    def options(self, request):
        return {"k": request.param}

    @pytest.mark.parametrize("static", [True, False])
    @pytest.mark.parametrize("k", [1, 2])
    def test_compute_gbest_return_values(
        self, swarm, options, topology, k, static
    ):
        """Test if update_gbest_neighborhood gives the expected return values"""
        topo = topology(static=static)
        expected_cost = 1.0002528364353296
        expected_pos = np.array([9.90438476e-01, 2.50379538e-03, 1.87405987e-05])
        expected_pos_2 = np.array([9.98033031e-01, 4.97392619e-03, 3.07726256e-03])
        pos, cost = topo.compute_gbest(swarm, **options)
        assert cost == pytest.approx(expected_cost)
        assert ((pos[np.argmin(cost)] == pytest.approx(expected_pos)) or
        (pos[np.argmin(cost)] == pytest.approx(expected_pos_2)))

    @pytest.mark.parametrize("static", [True, False])
    @pytest.mark.parametrize("k", [1, 2])
    def test_compute_neighbors_return_values(self, swarm, topology, k, static):
        """Test if __compute_neighbors() gives the expected shape and symmetry"""
        topo = topology(static=static)
        adj_matrix = topo._Random__compute_neighbors(swarm, k=k)
        assert adj_matrix.shape == (swarm.n_particles, swarm.n_particles)
        assert np.allclose(
            adj_matrix, adj_matrix.T, atol=1e-8
        )  # Symmetry test

    @pytest.mark.parametrize("static", [True, False])
    @pytest.mark.parametrize("k", [1])
    def test_compute_neighbors_adjacency_matrix(
        self, swarm, topology, k, static
    ):
        """Test if __compute_neighbors() gives the expected matrix"""
        np.random.seed(1)
        topo = topology(static=static)
        adj_matrix = topo._Random__compute_neighbors(swarm, k=k)
        # fmt: off
        comparison_matrix = np.array([[1, 1, 1, 0, 1, 0, 0, 0, 0, 1],
                                      [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
                                      [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
                                      [0, 1, 1, 1, 1, 0, 0, 0, 1, 0],
                                      [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
                                      [0, 1, 1, 0, 1, 1, 0, 1, 0, 1],
                                      [0, 1, 1, 0, 1, 0, 1, 0, 1, 1],
                                      [0, 1, 1, 0, 1, 1, 0, 1, 1, 0],
                                      [0, 1, 1, 1, 1, 0, 1, 1, 1, 0],
                                      [1, 1, 1, 0, 1, 1, 1, 0, 0, 1]])
        assert np.allclose(adj_matrix, comparison_matrix, atol=1e-8)
        # fmt: on



"""
Handlers

This module provides Handler classes for the position as well as the velocity
of particles. This is necessary when boundary conditions are imposed on the PSO
algorithm. Particles that do not stay inside these boundary conditions have to
be handled by either adjusting their position after they left the bounded
search space or adjusting their velocity when it would position them outside
the search space. In particular, this approach is important if the optimium of
a function is near the boundaries.
For the following documentation let :math:`x_{i, t, d}` be the :math:`d` th
coordinate of the particle :math:`i` 's position vector at the time :math:`t`,
:math:`lb` the vector of the lower boundaries and :math:`ub` the vector of the
upper boundaries.
The algorithms in this module are adapted from [SH2010]

[SH2010] Sabine Helwig, "Particle Swarms for Constrained Optimization",
PhD thesis, Friedrich-Alexander Universität Erlangen-Nürnberg, 2010.
"""

import inspect
import logging

import numpy as np

from ..utils.reporter import Reporter


class HandlerMixin(object):
    """ A HandlerMixing class

    This class offers some basic functionality for the Handlers.
    """

    def _merge_dicts(self, *dict_args):
        """Backward-compatible helper method to combine two dicts"""
        result = {}
        for dictionary in dict_args:
            result.update(dictionary)
        return result

    def _out_of_bounds(self, position, bounds):
        """Helper method to find indices of out-of-bound positions

        This method finds the indices of the particles that are out-of-bound.
        """
        lb, ub = bounds
        greater_than_bound = np.nonzero(position > ub)
        lower_than_bound = np.nonzero(position < lb)
        return (lower_than_bound, greater_than_bound)

    def _get_all_strategies(self):
        """Helper method to automatically generate a dict of strategies"""
        return {
            k: v
            for k, v in inspect.getmembers(self, predicate=inspect.isroutine)
            if not k.startswith(("__", "_"))
        }


class BoundaryHandler(HandlerMixin):
    def __init__(self, strategy):
        """ A BoundaryHandler class

        This class offers a way to handle boundary conditions. It contains
        methods to repair particle positions outside of the defined boundaries.
        Following strategies are available for the handling:

        * Nearest:
            Reposition the particle to the nearest bound.
        * Random:
            Reposition the particle randomly in between the bounds.
        * Shrink:
            Shrink the velocity of the particle such that it lands on the
            bounds.
        * Reflective:
            Mirror the particle position from outside the bounds to inside the
            bounds.
        * Intermediate:
            Reposition the particle to the midpoint between its current
            position on the bound surpassing axis and the bound itself.  This
            only adjusts the axes that surpass the boundaries.

        The BoundaryHandler can be called as a function to use the strategy
        that is passed at initialization to repair boundary issues. An example
        for the usage:

        .. code-block :: python

            from pyswarms.backend import operators as op
            from pyswarms.backend.handlers import BoundaryHandler

            bh = BoundaryHandler(strategy="reflective")
            ops.compute_position(swarm, bounds, handler=bh)

        By passing the handler, the :func:`compute_position()` function now has
        the ability to reset the particles by calling the :code:`BoundaryHandler`
        inside.

        Attributes
        ----------
        strategy : str
            The strategy to use. To see all available strategies,
            call :code:`BoundaryHandler.strategies`
        """
        self.strategy = strategy
        self.strategies = self._get_all_strategies()
        self.rep = Reporter(logger=logging.getLogger(__name__))
        self.memory = None

    def __call__(self, position, bounds, **kwargs):
        """Apply the selected strategy to the position-matrix given the bounds

        Parameters
        ----------
        position : numpy.ndarray
            The swarm position to be handled
        bounds : tuple of numpy.ndarray or list
            a tuple of size 2 where the first entry is the minimum bound while
            the second entry is the maximum bound. Each array must be of shape
            :code:`(dimensions,)`
        kwargs : dict

        Returns
        -------
        numpy.ndarray
            the adjusted positions of the swarm
        """
        try:
            new_position = self.strategies[self.strategy](
                position, bounds, **kwargs
            )
        except KeyError:
            message = "Unrecognized strategy: {}. Choose one among: " + str(
                [strat for strat in self.strategies.keys()]
            )
            self.rep.logger.exception(message.format(self.strategy))
            raise
        else:
            return new_position

    def nearest(self, position, bounds, **kwargs):
        """Set position to nearest bound

        This method resets particles that exceed the bounds to the nearest
        available boundary. For every axis on which the coordiantes of the particle
        surpasses the boundary conditions the coordinate is set to the respective
        bound that it surpasses.
        The following equation describes this strategy:

        .. math::

            x_{i, t, d} = \begin{cases}
                                lb_d & \quad \text{if }x_{i, t, d} < lb_d \\
                                ub_d & \quad \text{if }x_{i, t, d} > ub_d \\
                                x_{i, t, d} & \quad \text{otherwise}
                          \end{cases}

        """
        lb, ub = bounds
        bool_greater = position > ub
        bool_lower = position < lb
        new_pos = np.where(bool_lower, lb, position)
        new_pos = np.where(bool_greater, ub, new_pos)
        return new_pos

    def reflective(self, position, bounds, **kwargs):
        """Reflect the particle at the boundary

        This method reflects the particles that exceed the bounds at the
        respective boundary. This means that the amount that the component
        which is orthogonal to the exceeds the boundary is mirrored at the
        boundary. The reflection is repeated until the position of the particle
        is within the boundaries. The following algorithm describes the
        behaviour of this strategy:

        .. math::
            :nowrap:

            \begin{gather*}
                \text{while } x_{i, t, d} \not\in \left[lb_d,\,ub_d\right] \\
                \text{ do the following:}\\
                \\
                x_{i, t, d} =   \begin{cases}
                                    2\cdot lb_d - x_{i, t, d} & \quad \text{if } x_{i,
                                    t, d} < lb_d \\
                                    2\cdot ub_d - x_{i, t, d} & \quad \text{if } x_{i,
                                    t, d} > ub_d \\
                                    x_{i, t, d} & \quad \text{otherwise}
                                \end{cases}
            \end{gather*}
        """
        lb, ub = bounds
        lower_than_bound, greater_than_bound = self._out_of_bounds(
            position, bounds
        )
        new_pos = position
        while lower_than_bound[0].size != 0 or greater_than_bound[0].size != 0:
            if lower_than_bound[0].size > 0:
                new_pos[lower_than_bound] = (
                    2 * lb[lower_than_bound[1]] - new_pos[lower_than_bound]
                )
            if greater_than_bound[0].size > 0:
                new_pos[greater_than_bound] = (
                    2 * ub[greater_than_bound[1]] - new_pos[greater_than_bound]
                )
            lower_than_bound, greater_than_bound = self._out_of_bounds(
                new_pos, bounds
            )

        return new_pos

    def shrink(self, position, bounds, **kwargs):
        """Set the particle to the boundary

        This method resets particles that exceed the bounds to the intersection
        of its previous velocity and the boundary. This can be imagined as shrinking
        the previous velocity until the particle is back in the valid search space.
        Let :math:`\sigma_{i, t, d}` be the :math:`d` th shrinking value of the
        :math:`i` th particle at the time :math:`t` and :math:`v_{i, t}` the velocity
        of the :math:`i` th particle at the time :math:`t`. Then the new position is
        computed by the following equation:

        .. math::
            :nowrap:

            \begin{gather*}
            \mathbf{x}_{i, t} = \mathbf{x}_{i, t-1} + \sigma_{i, t} \mathbf{v}_{i, t} \\
            \\
            \text{with} \\
            \\
            \sigma_{i, t, d} = \begin{cases}
                                \frac{lb_d-x_{i, t-1, d}}{v_{i, t, d}} & \quad \text{if } x_{i, t, d} < lb_d \\
                                \frac{ub_d-x_{i, t-1, d}}{v_{i, t, d}} & \quad \text{if } x_{i, t, d} > ub_d \\
                                1 & \quad \text{otherwise}
                          \end{cases} \\
            \\
            \text{and} \\
            \\
            \sigma_{i, t} = \min_{d=1...n} \sigma_{i, t, d}
            \\
            \end{gather*}

        """
        if self.memory is None:
            new_pos = position
            self.memory = new_pos
        else:
            lb, ub = bounds
            lower_than_bound, greater_than_bound = self._out_of_bounds(
                position, bounds
            )
            velocity = position - self.memory
            # Create a coefficient matrix
            sigma = np.tile(1.0, position.shape)
            sigma[lower_than_bound] = (
                lb[lower_than_bound[1]] - self.memory[lower_than_bound]
            ) / velocity[lower_than_bound]
            sigma[greater_than_bound] = (
                ub[greater_than_bound[1]] - self.memory[greater_than_bound]
            ) / velocity[greater_than_bound]
            min_sigma = np.amin(sigma, axis=1)
            new_pos = position
            new_pos[lower_than_bound[0]] = (
                self.memory[lower_than_bound[0]]
                + np.multiply(
                    min_sigma[lower_than_bound[0]],
                    velocity[lower_than_bound[0]].T,
                ).T
            )
            new_pos[greater_than_bound[0]] = (
                self.memory[greater_than_bound[0]]
                + np.multiply(
                    min_sigma[greater_than_bound[0]],
                    velocity[greater_than_bound[0]].T,
                ).T
            )
            self.memory = new_pos
        return new_pos

    def random(self, position, bounds, **kwargs):
        """Set position to random location

        This method resets particles that exeed the bounds to a random position
        inside the boundary conditions.
        """
        lb, ub = bounds
        lower_than_bound, greater_than_bound = self._out_of_bounds(
            position, bounds
        )
        # Set indices that are greater than bounds
        new_pos = position
        new_pos[greater_than_bound[0]] = np.array(
            [
                np.array([u - l for u, l in zip(ub, lb)])
                * np.random.random_sample((position.shape[1],))
                + lb
            ]
        )
        new_pos[lower_than_bound[0]] = np.array(
            [
                np.array([u - l for u, l in zip(ub, lb)])
                * np.random.random_sample((position.shape[1],))
                + lb
            ]
        )
        return new_pos

    def intermediate(self, position, bounds, **kwargs):
        r"""Set the particle to an intermediate position

        This method resets particles that exceed the bounds to an intermediate
        position between the boundary and their earlier position. Namely, it changes
        the coordinate of the out-of-bounds axis to the middle value between the
        previous position and the boundary of the axis.
        The following equation describes this strategy:

        .. math::

            x_{i, t, d} = \begin{cases}
                                \frac{1}{2} \left (x_{i, t-1, d} + lb_d \right) & \quad \text{if }x_{i, t, d} < lb_d \\
                                \frac{1}{2} \left (x_{i, t-1, d} + ub_d \right) & \quad \text{if }x_{i, t, d} > ub_d \\
                                x_{i, t, d} & \quad \text{otherwise}
                          \end{cases}

        """
        if self.memory is None:
            new_pos = position
            self.memory = new_pos
        else:
            lb, ub = bounds
            lower_than_bound, greater_than_bound = self._out_of_bounds(
                position, bounds
            )
            new_pos = position
            new_pos[lower_than_bound] = 0.5 * (
                self.memory[lower_than_bound] + lb[lower_than_bound[1]]
            )
            new_pos[greater_than_bound] = 0.5 * (
                self.memory[greater_than_bound] + ub[greater_than_bound[1]]
            )
            self.memory = new_pos
        return new_pos

    def periodic(self, position, bounds, **kwargs):
        r"""Sets the particles a periodic fashion

        This method resets the particles that exeed the bounds by using the
        modulo function to cut down the position. This creates a virtual,
        periodic plane which is tiled with the search space.
        The following equation describtes this strategy:

        .. math::
            :nowrap:

            \begin{gather*}
            x_{i, t, d} = \begin{cases}
                                ub_d - (lb_d - x_{i, t, d}) \mod s_d & \quad \text{if }x_{i, t, d} < lb_d \\
                                lb_d + (x_{i, t, d} - ub_d) \mod s_d & \quad \text{if }x_{i, t, d} > ub_d \\
                                x_{i, t, d} & \quad \text{otherwise}
                          \end{cases}\\
            \\
            \text{with}\\
            \\
            s_d = |ub_d - lb_d|
            \end{gather*}

        """
        lb, ub = bounds
        lower_than_bound, greater_than_bound = self._out_of_bounds(
            position, bounds
        )
        bound_d = np.tile(
            np.abs(np.array(ub) - np.array(lb)), (position.shape[0], 1)
        )
        ub = np.tile(ub, (position.shape[0], 1))
        lb = np.tile(lb, (position.shape[0], 1))
        new_pos = position
        if lower_than_bound[0].size != 0 and lower_than_bound[1].size != 0:
            new_pos[lower_than_bound] = ub[lower_than_bound] - np.mod(
                (lb[lower_than_bound] - new_pos[lower_than_bound]),
                bound_d[lower_than_bound],
            )
        if greater_than_bound[0].size != 0 and greater_than_bound[1].size != 0:
            new_pos[greater_than_bound] = lb[greater_than_bound] + np.mod(
                (new_pos[greater_than_bound] - ub[greater_than_bound]),
                bound_d[greater_than_bound],
            )
        return new_pos


class VelocityHandler(HandlerMixin):
    def __init__(self, strategy):
        """ A VelocityHandler class

        This class offers a way to handle velocities. It contains
        methods to repair the velocities of particles that exceeded the
        defined boundaries. Following strategies are available for the handling:

        * Unmodified:
            Returns the unmodified velocites.
        * Adjust
            Returns the velocity that is adjusted to be the distance between the current
            and the previous position.
        * Invert
            Inverts and shrinks the velocity by the factor :code:`-z`.
        * Zero
            Sets the velocity of out-of-bounds particles to zero.

        """
        self.strategy = strategy
        self.strategies = self._get_all_strategies()
        self.rep = Reporter(logger=logging.getLogger(__name__))
        self.memory = None

    def __call__(self, velocity, clamp, **kwargs):
        """Apply the selected strategy to the velocity-matrix given the bounds

        Parameters
        ----------
        velocity : numpy.ndarray
            The swarm position to be handled
        clamp : tuple of numpy.ndarray or list
            a tuple of size 2 where the first entry is the minimum clamp while
            the second entry is the maximum clamp. Each array must be of shape
            :code:`(dimensions,)`
        kwargs : dict

        Returns
        -------
        numpy.ndarray
            the adjusted positions of the swarm
        """
        try:
            new_position = self.strategies[self.strategy](
                velocity, clamp, **kwargs
            )
        except KeyError:
            message = "Unrecognized strategy: {}. Choose one among: " + str(
                [strat for strat in self.strategies.keys()]
            )
            self.rep.logger.exception(message.format(self.strategy))
            raise
        else:
            return new_position

    def __apply_clamp(self, velocity, clamp):
        """Helper method to apply a clamp to a velocity vector"""
        clamped_vel = velocity
        min_velocity, max_velocity = clamp
        lower_than_clamp = clamped_vel <= min_velocity
        greater_than_clamp = clamped_vel >= max_velocity
        clamped_vel = np.where(lower_than_clamp, min_velocity, clamped_vel)
        clamped_vel = np.where(greater_than_clamp, max_velocity, clamped_vel)
        return clamped_vel

    def unmodified(self, velocity, clamp=None, **kwargs):
        """Leaves the velocity unchanged"""
        if clamp is None:
            new_vel = velocity
        else:
            if clamp is not None:
                new_vel = self.__apply_clamp(velocity, clamp)
        return new_vel

    def adjust(self, velocity, clamp=None, **kwargs):
        r"""Adjust the velocity to the new position

        The velocity is adjusted such that the following equation holds:
        .. math::

                \mathbf{v_{i,t}} = \mathbf{x_{i,t}} - \mathbf{x_{i,t-1}}

        .. note::
            This method should only be used in combination with a position handling
            operation.

        """
        try:
            if self.memory is None:
                new_vel = velocity
                self.memory = kwargs["position"]
            else:
                new_vel = kwargs["position"] - self.memory
                self.memory = kwargs["position"]
                if clamp is not None:
                    new_vel = self.__apply_clamp(new_vel, clamp)
        except KeyError:
            self.rep.logger.exception("Keyword 'position' missing")
            raise
        else:
            return new_vel

    def invert(self, velocity, clamp=None, **kwargs):
        """Invert the velocity if the particle is out of bounds

        The velocity is inverted and shrinked. The shrinking is determined by the
        kwarg :code:`z`. The default shrinking factor is :code:`0.5`. For all
        velocities whose particles are out of bounds the following equation is
        applied:
        .. math::

            \mathbf{v_{i,t}} = -z\mathbf{v_{i,t}}
        """
        try:
            # Default for the shrinking factor
            if "z" not in kwargs:
                z = 0.5
            else:
                z = kwargs["z"]
            lower_than_bound, greater_than_bound = self._out_of_bounds(
                kwargs["position"], kwargs["bounds"]
            )
            new_vel = velocity
            new_vel[lower_than_bound[0]] = (-z) * new_vel[lower_than_bound[0]]
            new_vel[greater_than_bound[0]] = (-z) * new_vel[
                greater_than_bound[0]
            ]
            if clamp is not None:
                new_vel = self.__apply_clamp(new_vel, clamp)
        except KeyError:
            self.rep.logger.exception("Keyword 'position' or 'bounds' missing")
            raise
        else:
            return new_vel

    def zero(self, velocity, clamp=None, **kwargs):
        """Set velocity to zero if the particle is out of bounds"""
        try:
            lower_than_bound, greater_than_bound = self._out_of_bounds(
                kwargs["position"], kwargs["bounds"]
            )
            new_vel = velocity
            new_vel[lower_than_bound[0]] = np.zeros(velocity.shape[1])
            new_vel[greater_than_bound[0]] = np.zeros(velocity.shape[1])
        except KeyError:
            self.rep.logger.exception("Keyword 'position' or 'bounds' missing")
            raise
        else:
            return new_vel



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import numpy as np
import pytest

# Import from pyswarms
import pyswarms.backend as P
from pyswarms.backend.handlers import BoundaryHandler, VelocityHandler


class TestComputePbest(object):
    """Test suite for compute_pbest()"""

    def test_return_values(self, swarm):
        """Test if method gives the expected return values"""
        expected_cost = np.array([1, 2, 2])
        expected_pos = np.array([[1, 2, 3], [4, 5, 6], [1, 1, 1]])
        pos, cost = P.compute_pbest(swarm)
        assert (pos == expected_pos).all()
        assert (cost == expected_cost).all()

    @pytest.mark.parametrize("swarm", [0, (1, 2, 3)])
    def test_input_swarm(self, swarm):
        """Test if method raises AttributeError with wrong swarm"""
        with pytest.raises(AttributeError):
            P.compute_pbest(swarm)


class TestComputeVelocity(object):
    """Test suite for compute_velocity()"""

    @pytest.mark.parametrize("clamp", [None, (0, 1), (-1, 1)])
    def test_return_values(self, swarm, clamp):
        """Test if method gives the expected shape and range"""
        vh = VelocityHandler(strategy="unmodified")
        v = P.compute_velocity(swarm, clamp, vh)
        assert v.shape == swarm.position.shape
        if clamp is not None:
            assert (clamp[0] <= v).all() and (clamp[1] >= v).all()

    @pytest.mark.parametrize("swarm", [0, (1, 2, 3)])
    @pytest.mark.parametrize("vh_strat", ["unmodified", "zero", "invert", "adjust"])
    def test_input_swarm(self, swarm, vh_strat):
        """Test if method raises AttributeError with wrong swarm"""
        vh = VelocityHandler(strategy=vh_strat)
        with pytest.raises(AttributeError):
            P.compute_velocity(swarm, clamp=(0, 1), vh=vh)

    @pytest.mark.parametrize("options", [{"c1": 0.5, "c2": 0.3}])
    @pytest.mark.parametrize("vh_strat", ["unmodified", "zero", "invert", "adjust"])
    def test_missing_kwargs(self, swarm, options, vh_strat):
        """Test if method raises KeyError with missing kwargs"""
        vh = VelocityHandler(strategy=vh_strat)
        with pytest.raises(KeyError):
            swarm.options = options
            clamp = (0, 1)
            P.compute_velocity(swarm, clamp, vh)


class TestComputePosition(object):
    """Test suite for compute_position()"""

    @pytest.mark.parametrize(
        "bounds",
        [None, ([-5, -5, -5], [5, 5, 5]), ([-10, -10, -10], [10, 10, 10])],
    )
    @pytest.mark.parametrize("bh_strat", ["nearest", "random"])
    def test_return_values(self, swarm, bounds, bh_strat):
        """Test if method gives the expected shape and range"""
        bh = BoundaryHandler(strategy=bh_strat)
        p = P.compute_position(swarm, bounds, bh)
        assert p.shape == swarm.velocity.shape
        if bounds is not None:
            assert (bounds[0] <= p).all() and (bounds[1] >= p).all()
            
    @pytest.mark.parametrize("swarm", [0, (1, 2, 3)])
    @pytest.mark.parametrize("bh_strat", ["nearest", "random", "shrink",
    "intermediate"])
    def test_input_swarm(self, swarm, bh_strat):
        """Test if method raises AttributeError with wrong swarm"""
        bh = BoundaryHandler(strategy=bh_strat)
        with pytest.raises(AttributeError):
            P.compute_position(swarm, bounds=([-5, -5], [5, 5]), bh=bh)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.backend.topology import VonNeumann
from .abc_test_topology import ABCTestTopology

np.random.seed(4135157)


class TestVonNeumannTopology(ABCTestTopology):
    @pytest.fixture
    def topology(self):
        return VonNeumann

    @pytest.fixture
    def options(self):
        return {"p": 1, "r": 1}

    @pytest.mark.parametrize("r", [0, 1])
    @pytest.mark.parametrize("p", [1, 2])
    def test_update_gbest_neighborhood(self, swarm, topology, p, r):
        """Test if update_gbest_neighborhood gives the expected return values"""
        topo = topology()
        pos, cost = topo.compute_gbest(swarm, p=p, r=r)
        expected_pos = np.array([9.90438476e-01, 2.50379538e-03, 1.87405987e-05])
        expected_pos_2 = np.array([9.98033031e-01, 4.97392619e-03, 3.07726256e-03])
        expected_cost = 1.0002528364353296
        assert cost == pytest.approx(expected_cost)
        assert ((pos[np.argmin(cost)] == pytest.approx(expected_pos)) or
                (pos[np.argmin(cost)] == pytest.approx(expected_pos_2)))

    @pytest.mark.parametrize("m", [i for i in range(3)])
    @pytest.mark.parametrize("n", [i for i in range(3)])
    def test_delannoy_numbers(self, m, n):
        expected_values = np.array([1, 3, 5, 7, 9, 11, 13, 15, 17])
        assert VonNeumann.delannoy(m, n) in expected_values



# -*- coding: utf-8 -*-

"""
Base class for Topologies

You can use this class to create your own topology. Note that every Topology
should implement a way to compute the (1) best particle, the (2) next
position, and the (3) next velocity given the Swarm's attributes at a given
timestep. Not implementing these methods will raise an error.

In addition, this class must interface with any class found in the
:mod:`pyswarms.backend.swarms.Swarm` module.
"""

# Import standard library
import abc
import logging

from ...utils.reporter import Reporter


class Topology(abc.ABC):
    def __init__(self, static, **kwargs):
        """Initializes the class"""

        # Initialize logger
        self.rep = Reporter(logger=logging.getLogger(__name__))

        # Initialize attributes
        self.static = static
        self.neighbor_idx = None

        if self.static:
            self.rep.log(
                "Running on `dynamic` topology,"
                "set `static=True` for fixed neighbors.",
                lvl=logging.DEBUG,
            )

    @abc.abstractmethod
    def compute_gbest(self, swarm):
        """Compute the best particle of the swarm and return the cost and
        position"""
        raise NotImplementedError("Topology::compute_gbest()")

    @abc.abstractmethod
    def compute_position(self, swarm):
        """Update the swarm's position-matrix"""
        raise NotImplementedError("Topology::compute_position()")

    @abc.abstractmethod
    def compute_velocity(self, swarm):
        """Update the swarm's velocity-matrix"""
        raise NotImplementedError("Topology::compute_velocity()")



# -*- coding: utf-8 -*-

"""
A Star Network Topology

This class implements a star topology. In this topology,
all particles are connected to one another. This social
behavior is often found in GlobalBest PSO
optimizers.
"""

# Import standard library
import logging

# Import modules
import numpy as np

from .. import operators as ops
from ..handlers import BoundaryHandler, VelocityHandler
from ...utils.reporter import Reporter
from .base import Topology


class Star(Topology):
    def __init__(self, static=None, **kwargs):
        # static = None is just an artifact to make the API consistent
        # Setting it will not change swarm behavior
        super(Star, self).__init__(static=True)
        self.rep = Reporter(logger=logging.getLogger(__name__))

    def compute_gbest(self, swarm, **kwargs):
        """Update the global best using a star topology

        This method takes the current pbest_pos and pbest_cost, then returns
        the minimum cost and position from the matrix.

        .. code-block:: python

            import pyswarms.backend as P
            from pyswarms.backend.swarms import Swarm
            from pyswarm.backend.topology import Star

            my_swarm = P.create_swarm(n_particles, dimensions)
            my_topology = Star()

            # Update best_cost and position
            swarm.best_pos, swarm.best_cost = my_topology.compute_gbest(my_swarm)

        Parameters
        ----------
        swarm : pyswarms.backend.swarm.Swarm
            a Swarm instance

        Returns
        -------
        numpy.ndarray
            Best position of shape :code:`(n_dimensions, )`
        float
            Best cost
        """
        try:
            if self.neighbor_idx is None:
                self.neighbor_idx = np.tile(
                    np.arange(swarm.n_particles), (swarm.n_particles, 1)
                )
            if np.min(swarm.pbest_cost) < swarm.best_cost:
                # Get the particle position with the lowest pbest_cost
                # and assign it to be the best_pos
                best_pos = swarm.pbest_pos[np.argmin(swarm.pbest_cost)]
                best_cost = np.min(swarm.pbest_cost)
            else:
                # Just get the previous best_pos and best_cost
                best_pos, best_cost = swarm.best_pos, swarm.best_cost
        except AttributeError:
            self.rep.logger.exception(
                "Please pass a Swarm class. You passed {}".format(type(swarm))
            )
            raise
        else:
            return (best_pos, best_cost)

    def compute_velocity(
        self,
        swarm,
        clamp=None,
        vh=VelocityHandler(strategy="unmodified"),
        bounds=None,
    ):
        """Compute the velocity matrix

        This method updates the velocity matrix using the best and current
        positions of the swarm. The velocity matrix is computed using the
        cognitive and social terms of the swarm.

        A sample usage can be seen with the following:

        .. code-block :: python

            import pyswarms.backend as P
            from pyswarms.backend.swarm import Swarm
            from pyswarms.backend.handlers import VelocityHandler
            from pyswarms.backend.topology import Star

            my_swarm = P.create_swarm(n_particles, dimensions)
            my_topology = Star()
            my_vh = VelocityHandler(strategy="adjust")

            for i in range(iters):
                # Inside the for-loop
                my_swarm.velocity = my_topology.update_velocity(my_swarm, clamp, my_vh,
                bounds)

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        clamp : tuple of floats (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        vh : pyswarms.backend.handlers.VelocityHandler
            a VelocityHandler instance
        bounds : tuple of :code:`np.ndarray` or list (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum bound while
            the second entry is the maximum bound. Each array must be of shape
            :code:`(dimensions,)`.

        Returns
        -------
        numpy.ndarray
            Updated velocity matrix
        """
        return ops.compute_velocity(swarm, clamp, vh, bounds=bounds)

    def compute_position(
        self, swarm, bounds=None, bh=BoundaryHandler(strategy="periodic")
    ):
        """Update the position matrix

        This method updates the position matrix given the current position and
        the velocity. If bounded, it waives updating the position.

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        bounds : tuple of :code:`np.ndarray` or list (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum bound while
            the second entry is the maximum bound. Each array must be of shape
            :code:`(dimensions,)`.
        bh : pyswarms.backend.handlers.BoundaryHandler
            a BoundaryHandler instance

        Returns
        -------
        numpy.ndarray
            New position-matrix
        """
        return ops.compute_position(swarm, bounds, bh)



# Import modules
import numpy as np
import pytest


@pytest.fixture()
def particles():
    shape = (np.random.randint(10, 20), np.random.randint(2, 6))
    particles_ = np.random.uniform(0, 10, shape)
    print(particles_)
    return particles_



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import standard library
import inspect

# Import modules
import numpy as np
import pytest

# Import from pyswarms
import pyswarms.backend.topology as t
from pyswarms.single import GeneralOptimizerPSO
from pyswarms.utils.functions.single_obj import sphere

from .abc_test_optimizer import ABCTestOptimizer


def istopology(x):
    """Helper predicate to check if it's a subclass"""
    return inspect.isclass(x) and not inspect.isabstract(x)


# Get all classes in the topology module, then
# Instatiate topologies, no need to suppy static param
topologies = [topo() for _, topo in inspect.getmembers(t, istopology)]


class TestGeneralOptimizer(ABCTestOptimizer):
    @pytest.fixture(params=topologies)
    def optimizer(self, request, options):
        x_max = 10 * np.ones(2)
        x_min = -1 * x_max
        bounds = (x_min, x_max)
        return GeneralOptimizerPSO(
            n_particles=10,
            dimensions=2,
            options=options,
            bounds=bounds,
            topology=request.param,
        )

    @pytest.fixture(params=topologies)
    def optimizer_history(self, request, options):
        opt = GeneralOptimizerPSO(
            n_particles=10,
            dimensions=2,
            options=options,
            topology=request.param,
        )
        opt.optimize(sphere, 1000)
        return opt

    @pytest.fixture(params=topologies)
    def optimizer_reset(self, request, options):
        opt = GeneralOptimizerPSO(
            n_particles=10,
            dimensions=2,
            options=options,
            topology=request.param,
        )
        opt.optimize(sphere, 1000)
        opt.reset()
        return opt

    def test_ftol_effect(self, optimizer):
        """Test if setting the ftol breaks the optimization process"""
        # Set optimizer tolerance
        optimizer.ftol = 1e-1
        optimizer.optimize(sphere, 2000)
        assert np.array(optimizer.cost_history).shape != (2000,)

    def test_parallel_evaluation(self, obj_without_args, optimizer):
        """Test if parallelization breaks the optimization process"""
        import multiprocessing
        optimizer.optimize(obj_without_args, 2000, n_processes=multiprocessing.cpu_count())
        assert np.array(optimizer.cost_history).shape == (2000,)

    @pytest.mark.skip(reason="Some topologies converge too slowly")
    def test_obj_with_kwargs(self, obj_with_args, optimizer):
        """Test if kwargs are passed properly in objfunc"""
        cost, pos = optimizer.optimize(obj_with_args, 1000, a=1, b=100)
        assert np.isclose(cost, 0, rtol=1e-03)
        assert np.isclose(pos[0], 1.0, rtol=1e-03)
        assert np.isclose(pos[1], 1.0, rtol=1e-03)

    def test_obj_unnecessary_kwargs(self, obj_without_args, optimizer):
        """Test if error is raised given unnecessary kwargs"""
        with pytest.raises(TypeError):
            # kwargs `a` should not be supplied
            cost, pos = optimizer.optimize(obj_without_args, 1000, a=1)

    def test_obj_missing_kwargs(self, obj_with_args, optimizer):
        """Test if error is raised with incomplete kwargs"""
        with pytest.raises(TypeError):
            # kwargs `b` is missing here
            cost, pos = optimizer.optimize(obj_with_args, 1000, a=1)

    def test_obj_incorrect_kwargs(self, obj_with_args, optimizer):
        """Test if error is raised with wrong kwargs"""
        with pytest.raises(TypeError):
            # Wrong kwargs
            cost, pos = optimizer.optimize(obj_with_args, 1000, c=1, d=100)

    def test_general_correct_pos(self, options, optimizer):
        """ Test to check general optimiser returns the correct position corresponding to the best cost """
        cost, pos = optimizer.optimize(sphere, iters=5)
        # find best pos from history
        min_cost_idx = np.argmin(optimizer.cost_history)
        min_pos_idx = np.argmin(sphere(optimizer.pos_history[min_cost_idx]))
        assert np.array_equal(optimizer.pos_history[min_cost_idx][min_pos_idx], pos)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import numpy as np
import pytest

from .abc_test_optimizer import ABCTestOptimizer

# from pyswarms.utils.functions.single_obj import sphere


class ABCTestDiscreteOptimizer(ABCTestOptimizer):
    """Abstract class that defines various tests for high-level optimizers

    Whenever an optimizer implementation inherits from ABCTestOptimizer,
    you don't need to write down all tests anymore. Instead, you can just
    specify all required fixtures in the test suite.
    """

    @pytest.mark.skip("No way of testing this yet")
    def test_obj_with_kwargs(self, obj_with_args, optimizer, options):
        """Test if kwargs are passed properly in objfunc"""
        opt = optimizer(100, 2, options=options)
        cost, pos = opt.optimize(obj_with_args, 1000, a=1, b=100)
        assert np.isclose(cost, 0, rtol=1e-03)
        assert np.isclose(pos[0], 1.0, rtol=1e-03)
        assert np.isclose(pos[1], 1.0, rtol=1e-03)

    @pytest.mark.skip("No way of testing this yet")
    def test_obj_unnecessary_kwargs(
        self, obj_without_args, optimizer, options
    ):
        """Test if error is raised given unnecessary kwargs"""
        opt = optimizer(100, 2, options=options)
        with pytest.raises(TypeError):
            # kwargs `a` should not be supplied
            cost, pos = opt.optimize(obj_without_args, 1000, a=1)

    @pytest.mark.skip("No way of testing this yet")
    def test_obj_missing_kwargs(self, obj_with_args, optimizer, options):
        """Test if error is raised with incomplete kwargs"""
        opt = optimizer(100, 2, options=options)
        with pytest.raises(TypeError):
            # kwargs `b` is missing here
            cost, pos = opt.optimize(obj_with_args, 1000, a=1)

    @pytest.mark.skip("No way of testing this yet")
    def test_obj_incorrect_kwargs(self, obj_with_args, optimizer, options):
        """Test if error is raised with wrong kwargs"""
        opt = optimizer(100, 2, options=options)
        with pytest.raises(TypeError):
            # Wrong kwargs
            cost, pos = opt.optimize(obj_with_args, 1000, c=1, d=100)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

# Import modules
from setuptools import find_packages, setup

with open("README.md", encoding="utf8") as readme_file:
    readme = readme_file.read()

with open("requirements.in") as f:
    requirements = f.read().splitlines()

with open("requirements-dev.txt") as f:
    dev_requirements = f.read().splitlines()


setup(
    name="pyswarms",
    version="1.1.0",
    description="A Python-based Particle Swarm Optimization (PSO) library.",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Lester James V. Miranda",
    author_email="ljvmiranda@gmail.com",
    url="https://github.com/ljvmiranda921/pyswarms",
    packages=find_packages(exclude=["docs", "tests"]),
    include_package_data=True,
    install_requires=requirements,
    tests_require=dev_requirements,
    extras_require={"test": dev_requirements},
    license="MIT license",
    zip_safe=False,
    keywords="pyswarms",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Topic :: Scientific/Engineering",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    test_suite="tests",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Fixtures for tests"""

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.single import LocalBestPSO
from pyswarms.utils.functions.single_obj import sphere

# Import from package
from pyswarms.utils.search.grid_search import GridSearch
from pyswarms.utils.search.random_search import RandomSearch


@pytest.fixture
def grid():
    """Returns a GridSearch instance"""
    options = {
        "c1": [1, 2, 3],
        "c2": [1, 2, 3],
        "k": [5, 10, 15],
        "w": [0.9, 0.7, 0.4],
        "p": [1],
    }
    return GridSearch(
        LocalBestPSO,
        n_particles=40,
        dimensions=20,
        options=options,
        objective_func=sphere,
        iters=10,
        bounds=None,
    )


@pytest.fixture
def grid_mini():
    """Returns a GridSearch instance with a smaller search-space"""
    options = {"c1": [1, 2], "c2": 6, "k": 5, "w": 0.9, "p": 0}
    return GridSearch(
        LocalBestPSO,
        n_particles=40,
        dimensions=20,
        options=options,
        objective_func=sphere,
        iters=10,
        bounds=None,
    )


@pytest.fixture
def random_unbounded():
    """Returns a RandomSearch instance without bounds"""
    options = {
        "c1": [1, 5],
        "c2": [6, 10],
        "k": [11, 15],
        "w": [0.4, 0.9],
        "p": 1,
    }
    return RandomSearch(
        LocalBestPSO,
        n_particles=40,
        dimensions=20,
        options=options,
        objective_func=sphere,
        iters=10,
        n_selection_iters=100,
        bounds=None,
    )


@pytest.fixture
def random_bounded():
    """Returns a RandomSearch instance with bounds"""
    bounds = (np.array([-5, -5]), np.array([5, 5]))
    options = {
        "c1": [1, 5],
        "c2": [6, 10],
        "k": [11, 15],
        "w": [0.4, 0.9],
        "p": 1,
    }
    return RandomSearch(
        LocalBestPSO,
        n_particles=40,
        dimensions=20,
        options=options,
        objective_func=sphere,
        iters=10,
        n_selection_iters=100,
        bounds=bounds,
    )



import pytest
import inspect
import numpy as np

from pyswarms.backend.handlers import (
    BoundaryHandler,
    VelocityHandler,
    HandlerMixin,
)
import pyswarms.backend.handlers as h

bh_strategies = [name for name, _ in inspect.getmembers(h.BoundaryHandler(""),
    predicate=inspect.ismethod) if not name.startswith(("__", "_"))]
vh_strategies = [name for name, _ in inspect.getmembers(h.VelocityHandler(""),
    predicate=inspect.ismethod) if not name.startswith(("__", "_"))]

def test_out_of_bounds(bounds, positions_inbound, positions_out_of_bound):
    hm = HandlerMixin()
    out_of_bounds = hm._out_of_bounds
    idx_inbound = out_of_bounds(positions_inbound, bounds)
    idx_out_of_bounds = out_of_bounds(positions_out_of_bound, bounds)

    print(bh_strategies)
    expected_idx = (
        (np.array([2, 3, 5]), np.array([1, 2, 0])),
        (np.array([0, 1, 2, 3, 4, 5]), np.array([0, 1, 1, 2, 1, 0])),
    )
    assert np.ravel(idx_inbound[0]).size == 0
    assert np.ravel(idx_inbound[1]).size == 0
    assert (
        np.ravel(idx_out_of_bounds[0]).all() == np.ravel(expected_idx[0]).all()
    )
    assert (
        np.ravel(idx_out_of_bounds[1]).all() == np.ravel(expected_idx[1]).all()
    )

@pytest.mark.parametrize(
    "strategy", bh_strategies
)
def test_bound_handling(
    bounds, positions_inbound, positions_out_of_bound, strategy
):
    bh = BoundaryHandler(strategy=strategy)
    # Test if it doesn't handle inbound positions
    inbound_handled = bh(positions_inbound, bounds)
    assert inbound_handled.all() == positions_inbound.all()

    # Test if all particles are handled to a position inside the boundaries
    outbound_handled = bh(positions_out_of_bound, bounds)
    lower_than_bound = outbound_handled >= bounds[0]
    greater_than_bound = outbound_handled <= bounds[1]
    assert lower_than_bound.all()
    assert greater_than_bound.all()

def test_nearest_strategy(bounds, positions_inbound, positions_out_of_bound):
    bh = BoundaryHandler(strategy="nearest")
    # TODO Add strategy specific tests


def test_reflective_strategy(
    bounds, positions_inbound, positions_out_of_bound
):
    bh = BoundaryHandler(strategy="reflective")
    pass
    # TODO Add strategy specific tests


def test_shrink_strategy(bounds, positions_inbound, positions_out_of_bound):
    bh = BoundaryHandler(strategy="shrink")
    # TODO Add strategy specific tests


def test_random_strategy(bounds, positions_inbound, positions_out_of_bound):
    bh = BoundaryHandler(strategy="random")
    # TODO Add strategy specific tests


def test_intermediate_strategy(
    bounds, positions_inbound, positions_out_of_bound
):
    bh = BoundaryHandler(strategy="intermediate")
    # TODO Add strategy specific tests


def test_periodic_strategy(bounds, positions_inbound, positions_out_of_bound):
    bh = BoundaryHandler(strategy="periodic")
    # TODO Add strategy specific tests

def assert_clamp(
clamp,
velocities_inbound,
velocities_out_of_bound,
positions_inbound,
positions_out_of_bound,
vh,
bounds=None,
):
# Test if it doesn't handle inclamp velocities
    inbound_handled = vh(
        velocities_inbound, clamp, position=positions_inbound, bounds=bounds
    )
    assert inbound_handled.all() == velocities_inbound.all()

    # Test if all particles are handled to a velocity inside the clamp
    outbound_handled = vh(
        velocities_out_of_bound,
        clamp,
        position=positions_out_of_bound,
        bounds=bounds,
    )
    lower_than_clamp = outbound_handled < clamp[0]
    greater_than_clamp = outbound_handled > clamp[1]
    assert not lower_than_clamp.all()
    assert not greater_than_clamp.all()




def test_unmodified_strategy(
    clamp, velocities_inbound, velocities_out_of_bound
):
    vh = VelocityHandler(strategy="unmodified")
    inbound_handled = vh(velocities_inbound, clamp)
    outbound_handled = vh(velocities_out_of_bound, clamp)
    assert inbound_handled.all() == velocities_inbound.all()
    assert outbound_handled.all() == velocities_out_of_bound.all()


def test_adjust_strategy(
    clamp,
    velocities_inbound,
    velocities_out_of_bound,
    positions_inbound,
    positions_out_of_bound,
):
    vh = VelocityHandler(strategy="adjust")
    assert_clamp(
        clamp,
        velocities_inbound,
        velocities_out_of_bound,
        positions_inbound,
        positions_out_of_bound,
        vh,
    )
    # TODO Add strategy specific tests
    pass


def test_invert_strategy(
    clamp,
    velocities_inbound,
    velocities_out_of_bound,
    positions_inbound,
    positions_out_of_bound,
    bounds,
):
    vh = VelocityHandler(strategy="invert")
    assert_clamp(
        clamp,
        velocities_inbound,
        velocities_out_of_bound,
        positions_inbound,
        positions_out_of_bound,
        vh,
        bounds=bounds,
    )
    # TODO Add strategy specific tests
    pass


def test_zero_strategy(
    clamp,
    velocities_inbound,
    velocities_out_of_bound,
    positions_inbound,
    positions_out_of_bound,
    bounds,
):
    vh = VelocityHandler(strategy="zero")
    # TODO Add strategy specific tests
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest


@pytest.mark.parametrize("maximum", [True, False])
def test_search_best_options_return_type(random_unbounded, maximum):
    """Tests if best options returns a dictionary"""
    _, best_options = random_unbounded.search(maximum)
    assert isinstance(best_options, dict)


def test_generate_grid_combinations(random_bounded):
    """Test that the number of combinations in grid equals the number
    parameter selection iterations specficied"""
    expected = 100
    grid = random_bounded.generate_grid()
    assert len(grid) == expected


@pytest.mark.parametrize("options", ["c1", "c2", "k", "w"])
def test_generate_grid_parameter_mapping(random_bounded, options):
    """Test that generated values are correctly mapped to each parameter and
    are within the specified bounds """
    grid = random_bounded.generate_grid()
    values = [x[options] for x in grid]
    for val in values:
        assert val >= random_bounded.options[options][0]
        assert val <= random_bounded.options[options][1]



"""
The :mod:`pyswarms.decorators` module implements a decorator that
can be used to simplify the task of writing the cost function for
an optimization run. The decorator can be directly called by using
:code:`@pyswarms.cost`.
"""
from .decorators import cost

__all__ = ["cost"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Fixtures for tests"""

# Import modules
import numpy as np
import pytest


@pytest.fixture
def outbound():
    """Returns a function that generates a matrix out of bounds a given
    range"""

    def _outbound(low, high, size, tol=1000, nums=100):
        """Generates a matrix that is out of bounds"""
        low_end = -np.random.uniform(tol, low, (nums,))
        high_end = np.random.uniform(tol, high, (nums,))
        choices = np.hstack([low_end, high_end])
        return np.random.choice(choices, size=size, replace=True)

    return _outbound


@pytest.fixture
def outdim():
    """Returns a matrix of bad shape (3D matrix)"""
    return np.zeros(shape=(3, 3))


@pytest.fixture
def common_minima():
    """Returns a zero-matrix with a common-minima for most objective
    functions"""
    return np.zeros(shape=(3, 2))


@pytest.fixture
def common_minima2():
    """Returns a one-matrix with a common-minima for most objective
    functions"""
    return np.ones(shape=(3, 2))


@pytest.fixture
def targetdim():
    """Returns a baseline target dimension for most objective functions"""
    return (3,)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pyswarms` package."""

# Import standard library
from collections import namedtuple

# Import modules
import numpy as np
import pytest

# Import from pyswarms
from pyswarms.utils.functions import single_obj as fx

Bounds = namedtuple("Bounds", "low high")
b = {
    # Define all bounds here
    "ackley": Bounds(low=-32, high=32),
    "beale": Bounds(low=-4.5, high=4.5),
    "booth": Bounds(low=-10, high=10),
    # bukin_6 is not symmetrical
    "crossintray": Bounds(low=-1, high=10),
    "easom": Bounds(low=-100, high=100),
    "eggholder": Bounds(low=-512, high=512),
    "goldstein": Bounds(low=-2, high=-2),
    "himmelblau": Bounds(low=-5, high=5),
    "holdertable": Bounds(low=-10, high=10),
    "levi": Bounds(low=-10, high=10),
    "matyas": Bounds(low=-10, high=10),
    "rastrigin": Bounds(low=-5.12, high=5.12),
    # rosenbrock has an infinite domain
    "schaffer2": Bounds(low=-100, high=100),
    "threehump": Bounds(low=-5, high=5),
}


def test_ackley_bound_fail(outbound):
    """Test ackley bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["ackley"].low, b["ackley"].high, size=(3, 2))
        fx.ackley(x)


def test_beale_bound_fail(outbound):
    """Test beale bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["beale"].low, b["beale"].high, size=(3, 2))
        fx.beale(x)


def test_booth_bound_fail(outbound):
    """Test booth bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["booth"].low, b["booth"].high, size=(3, 2))
        fx.booth(x)


@pytest.mark.parametrize(
    "x",
    [
        -np.random.uniform(15.001, 100, (3, 2)),
        np.random.uniform(-5.001, -3.001, (3, 2)),
        np.random.uniform(-3.001, -100, (3, 2)),
    ],
)
def test_bukin6_bound_fail(x):
    """Test bukin6 bound exception"""
    with pytest.raises(ValueError):
        fx.bukin6(x)


def test_crossintray_bound_fail(outbound):
    """Test crossintray bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["crossintray"].low, b["crossintray"].high, size=(3, 2))
        fx.crossintray(x)


def test_easom_bound_fail(outbound):
    """Test easom bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["easom"].low, b["easom"].high, size=(3, 2))
        fx.easom(x)


def test_eggholder_bound_fail(outbound):
    """Test eggholder bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["eggholder"].low, b["eggholder"].high, size=(3, 2))
        fx.eggholder(x)


def test_goldstein_bound_fail(outbound):
    """Test goldstein bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["goldstein"].low, b["goldstein"].high, size=(3, 2))
        fx.goldstein(x)


def test_himmelblau_bound_fail(outbound):
    """Test himmelblau bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["himmelblau"].low, b["himmelblau"].high, size=(3, 2))
        fx.himmelblau(x)


def test_holdertable_bound_fail(outbound):
    """Test holdertable bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["holdertable"].low, b["holdertable"].high, size=(3, 2))
        fx.holdertable(x)


def test_levi_bound_fail(outbound):
    """Test levi bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["levi"].low, b["levi"].high, size=(3, 2))
        fx.levi(x)


def test_matyas_bound_fail(outbound):
    """Test matyas bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["matyas"].low, b["matyas"].high, size=(3, 2))
        fx.matyas(x)


def test_rastrigin_bound_fail(outbound):
    """Test rastrigin bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["rastrigin"].low, b["rastrigin"].high, size=(3, 2))
        fx.rastrigin(x)


def test_schaffer2_bound_fail(outbound):
    """Test schaffer2 bound exception"""
    with pytest.raises(ValueError):
        x = outbound(
            b["schaffer2"].low, b["schaffer2"].high, tol=200, size=(3, 2)
        )
        fx.schaffer2(x)


def test_threehump_bound_fail(outbound):
    """Test threehump bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b["threehump"].low, b["threehump"].high, size=(3, 2))
        fx.threehump(x)



"""
The :mod:`pyswarms.single` module implements various techniques in
continuous single-objective optimization. These require only one
objective function that can be optimized in a continuous space.

.. note::
    PSO algorithms scale with the search space. This means that, by
    using larger boundaries, the final results are getting larger
    as well.

.. note::
    Please keep in mind that Python has a biggest float number.
    So using large boundaries in combination with exponentiation or
    multiplication can lead to an :code:`OverflowError`.
"""

from .global_best import GlobalBestPSO
from .local_best import LocalBestPSO
from .general_optimizer import GeneralOptimizerPSO

__all__ = ["GlobalBestPSO", "LocalBestPSO", "GeneralOptimizerPSO"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest


@pytest.mark.parametrize("maximum", [True, False])
def test_search_best_options_return_type(grid, maximum):
    """Tests if best options returns a dictionary"""
    _, best_options = grid.search(maximum)
    assert isinstance(best_options, dict)


def test_grid_output(grid_mini):
    """Tests if generate_grid function returns expected value"""
    expected = [
        {"c1": 1, "c2": 6, "k": 5, "w": 0.9, "p": 0},
        {"c1": 2, "c2": 6, "k": 5, "w": 0.9, "p": 0},
    ]
    assert grid_mini.generate_grid() == expected
